/*
 * Filename: NSApplication.h
 * Project:  NSXKit
 * Module:   Main application components
 *
 * Function: The NSApplication class manages the main event loop of
 *           the program, keeps track of the application's windows
 *           and which one is the key window.
 *
 * Author:   Pascal Forget
 * Date:     January 1996
 *
 * Copyright (C) 1996 Pascal Forget.  All Rights Reserved
 */

#ifndef NSAPPLICATION_H__
#define NSAPPLICATION_H__

#include <Foundation/NSObject.h>
#include "NSGlobal.h"
#include "NSImage.h"
#include "NSView.h"
#include "NSWindow.h"
#include "NSEvent.h"

#define BITMAPDEPTH 1
#define TOO_SMALL 0
#define BIG_ENOUGH 1

@interface NSApplication : NSObject
{
    NSImage *appIcon;
    id delegate;
    BOOL isActive;
    BOOL isHidden;
    BOOL isRunning;
    NSMutableArray *windows;
    BOOL windowsNeedUpdate;
    NSMutableArray *_eventQueue;  // keep events sent by postEvent:atStart:
}

/* Old stuff */

/*
- createSystemImages;
- loadObjectArchive:(const char *)path;
- quit;
*/

+ (NSApplication *)sharedApplication;
- (void)abortModal;
- (void)activateIgnoringOtherApps:(BOOL)flag;
- (NSModalSession)beginModalSessionForWindow:(NSWindow *)theWindow;
- (void)deactivate;
- (void)endModalSession:(NSModalSession)session;
- (void)finishLaunching;
- (BOOL)isActive;
- (BOOL)isRunning;
- (void)run;
- (int)runModalForWindow:(NSWindow *)theWindow;
- (int)runModalsession:(NSModalSession)session;
- (void)sendEvent:(NSEvent *)theEvent;
- (void)stop:(id)sender;
- (void)stopModal;
- (void)stopModalWithCode:(int)returnCode;
- (NSEvent *)currentEvent;
- (void)discardsEventsMatchingMask:(unsigned int)mask
		       beforeEvent:(NSEvent *)lastEvent;
- (NSEvent *)nextEventMatchingMask:(unsigned int)mask
			 untilDate:(NSDate *)expiration
			    inMode:(NSString *)mode
			   dequeue:(BOOL)flag;
- (void)postEvent:(NSEvent *)event atStart:(BOOL)flag;
- (BOOL)sendAction:(SEL)aSelector to:(id)aTarget from:(id)sender;
- (id)targetForAction:(SEL)aSelector;
- (BOOL)tryToPerform:(SEL)aSelector with:(id)anObject;
- (void)setApplicationIconImage:(NSImage *)anImage;
- (NSImage *)applicationIconImage;
- (void)hide:(id)sender;
- (BOOL)isHidden;
- (void)unhide:(id)sender;
- (void)unhideWithoutActivation;
- (NSWindow *)keyWindow;
- (NSWindow *)mainWindow;
- (NSWindow *)makeWindowsPerform:(SEL)aSelector inOrder:(BOOL)flag;
- (void)miniaturizeAll:(id)sender;
- (void)preventWindowOrdering;
- (void)setWindowsNeedUpdate:(BOOL)flag;
- (void)updateWindows;
- (NSArray *)windows;
- (NSWindow *)windowWithWindowNumber:(int)windowNum;
- (void)terminate:(id)sender;

@end

@interface NSApplication (NonGNUStep)
- (void)registerWindow:(NSWindow *)newWindow;
- (void)removeWindow:(NSWindow *)defunctWindow;
- (void)setKeyWindow:(NSWindow *)aWindow;
- (void)setMainWindow:(NSWindow *)aWindow;
- (void)updateKeyAndMainWindow;
@end

#endif
