/* 
   NSAttributedString.h

   Headerfile for AppKit additions for NSAttributedString class

   Copyright (C) 1997 Free Software Foundation, Inc.

   Written by: ANOQ of the Sun <anoq@vip.cybercity.dk>
   Date: August 1997
   
   This file is part of NSXKit

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _NSXKit_H_NSAttributedStringAdd
#define _NSXKit_H_NSAttributedStringAdd

#include <Foundation/NSAttributedString.h>
#include "NSTextAttachment.h"
#include "NSStringDrawing.h"

NSString *NSFontAttributeName;
NSString *NSForegroundColorAttributeName;
NSString *NSBackgroundColorAttributeName;
NSString *NSUnderlineStyleAttributeName;
NSString *NSSuperscriptAttributeName;
NSString *NSBaselineOffsetAttributeName;
NSString *NSKernAttributeName;
NSString *NSLigatureAttributeName;
NSString *NSParagraphStyleAttributeName;
NSString *NSAttachmentAttributeName;

@interface NSAttributedString (NSAttributedStringAdditions)

//Creating an NSAttributedString
-(id)initWithRTF:(NSData *)rtfData documentAttributes:(NSDictionary **)docAttributes;
-(id)initWithRTFD:(NSData *)rtfdData documentAttributes:(NSDictionary **)docAttributes;
-(id)initWithRTFDFileWrapper:(NSFileWrapper *)wrapper documentAttributes:(NSDictionary **)docAttributes;
-(id)initWithPath:(NSString *)path documentAttributes:(NSDictionary **)docAttributes;
+ (NSAttributedString *)attributedStringWithAttachment:(NSTextAttachment *)attachment;

//Retrieving attribute information
- (NSDictionary *)fontAttributesInRange:(NSRange)aRange;
- (NSDictionary *)rulerAttributesInRange:(NSRange)aRange;
- (BOOL)containsAttachments;

//Calculating linguistic units
- (NSRange)doubleClickAtIndex:(unsigned int)index;
- (unsigned int)lineBreakBeforeIndex:(unsigned int)index withinRange:(NSRange)aRange;
- (unsigned int)nextWordFromIndex:(unsigned int)index forward:(BOOL)flag;

//Drawing the string
- (void)__drawAtPoint:(NSPoint)point inRange:(NSRange)range;//Internal method to eliminate
                                                            //the need for extracting a
                                                            //substring when scrolling
                                                            //with NSText...
-(void)drawAtPoint:(NSPoint)point;
-(void)drawInRect:(NSRect)rect;
- (NSSize)size;

//Generating RTF data
- (NSData *)RTFFromRange:(NSRange)aRange documentAttributes:(NSDictionary *)docAttributes;
- (NSFileWrapper *)RTFDFileWrapperFromRange:(NSRange)aRange documentAttributes:(NSDictionary *)docAttributes;
- (NSData *)RTFDFromRange:(NSRange)aRange documentAttributes:(NSDictionary *)docAttributes;

@end //NSAttributedString (NSAttributedStringAdditions)

@interface NSMutableAttributedString (NSAttributedStringAdditions)

//Changing attributes
-(void)applyFontTraits:(NSFontTraitMask)mask range:(NSRange)aRange;
-(void)setAlignment:(NSTextAlignment)alignment range:(NSRange)aRange;
-(void)subscriptRange:(NSRange)aRange;
-(void)superscriptRange:(NSRange)aRange;
-(void)unscriptRange:(NSRange)aRange;

//Updating attachment contents
-(void)updateAttachmentsFromPath:(NSString *)path;

//Fixing attributes after changes
-(void)fixAttributesInRange:(NSRange)aRange;
-(void)fixAttachmentAttributeInRange:(NSRange)aRange;
-(void)fixFontAttributeInRange:(NSRange)aRange;
-(void)fixParagraphStyleAttributeInRange:(NSRange)aRange;

@end //NSMutableAttributedString

#endif //_NSXKit_H_NSAttributedStringAdd