/*
 * Filename: NSBox.h
 * Project:  Pascal's X Kit
 * Module:   Basic application components
 *
 * Function: A box is a simple view that draws a border
 *           around itself, with an optional title.
 *
 * Note:     The content view is stored at position zero
 *           in the subviews list
 *
 * Author:   Pascal Forget
 * Date:     December 1996
 *
 * Copyright (C) 1996 Pascal Forget. All Rights Reserved.
 */

#ifndef NSBOX_H__
#define NSBOX_H__

#include "NSView.h"
#include "NSCell.h"
#include "NSFont.h"

@interface NSBox : NSView
{
    NSRect borderRect;
    NSBorderType borderType;
    NSString *title;
    NSCell *titleCell;
    NSFont *titleFont;
    NSTitlePosition titlePosition;
    NSRect titleRect;
    NSSize contentViewMargins;
    NSView *contentView; // Same as [subviews object atIndex:0]
}

- (NSRect)borderRect;
- (NSBorderType)borderType;
- (id)initWithFrame:(NSRect)frameRect;
- (void)setBorderType:(NSBorderType)aType;
- (void)setTitle:(NSString *)aString;
- (void)setTitleFont:(NSFont *)fontObj;
- (void)setTitlePosition:(NSTitlePosition)aPosition;
- (NSString *)title;
- (id)titleCell;
- (NSFont *)titleFont;
- (NSTitlePosition)titlePosition;
- (NSRect)titleRect;
- (id)contentView;
- (NSSize)contentViewMargins;
- (void)setContentView:(NSView *)aView;
- (void)setContentViewMargins:(NSSize)offsetSize;
- (void)setFrameFromContentFrame:(NSRect)contentFrame;
- (void)sizeToFit;

@end
#endif
