/*
 * Filename: NSCell.h
 * Project:  Pascal's X Kit
 * Module:   Basic components
 *
 * Function: Basic Cell class.
 *
 * Author:   Pascal Forget
 * Date:     January 1997
 *
 * Copyright (C) 1997 Pascal Forget.  All Rights Reserved.
 */

#ifndef NSCELL_H__
#define NSCELL_H__

#include "NSGlobal.h"
#include "NSView.h"
#include "NSFont.h"
#include "NSImage.h"

@interface NSCell : NSObject
{
    NSTextAlignment cellAlignment;
    NSFont *cellFont;
    NSSize cellSize;
    NSCellType cellType;
    int entryType;
    unsigned int floatLeftDigits;
    unsigned int floatRightDigits;
    NSImage *image;
    BOOL isBezeled;
    BOOL isBordered;
    BOOL isContinuous;
    BOOL isDisabled;
    BOOL isEditable;
    BOOL isEnabled;
    BOOL isFloatAutoRange;
    BOOL isHighlighted;
    BOOL isOpaque;
    BOOL isScrollable;
    BOOL isSelectable;
    id representedObject;
    int state;
    NSString *stringValue;
    BOOL wraps;
}

+ (BOOL)prefersTrackingUntilMouseUp;
- (SEL)action;
- (NSTextAlignment)alignment;
- (void)calcDrawInfo:(NSRect)aRect;
- (NSSize)cellSize;
- (NSSize)cellSizeForBounds:(NSRect)aRect;
- (NSComparisonResult)compare:(id)otherCell;
- (NSRect)drawingRectForBounds:(NSRect)theRect;
- (NSView *)controlView;
- (double)doubleValue;
- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView;
- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView;
- (int)entryType;
- (float)floatValue;
- (NSFont *)font;
- (void)highlight:(BOOL)lit
	withFrame:(NSRect)cellFrame
	   inView:(NSView *)controlView;
- (NSImage *)image;
- (NSRect)imageRectForBounds:(NSRect)theRect;
- (id)initImageCell:(NSImage *)anImage;
- (id)initTextCell:(NSString *)aString;
- (int)intValue;
- (BOOL)isBezeled;
- (BOOL)isBordered;
- (BOOL)isContinuous;
- (BOOL)isEditable;
- (BOOL)isEnabled;
- (BOOL)isEntryAcceptable:(NSString *)aString;
- (BOOL)isHighlighted;
- (BOOL)isOpaque;
- (BOOL)isScrollable;
- (BOOL)isSelectable;
- (NSString *)keyEquivalent;
- (id)representedObject;
- (void)setAction:(SEL)aSelector;
- (void)setAlignment:(NSTextAlignment)mode;
- (void)setBezeled:(BOOL)flag;
- (void)setBordered:(BOOL)flag;
- (void)setContinuous:(BOOL)flag;
- (void)setDoubleValue:(double)aDouble;
- (void)setEditable:(BOOL)flag;
- (void)setEnabled:(BOOL)flag;
- (void)setEntryType:(int)aType;
- (void)setFloatingPointFormat:(BOOL)autoRange
			  left:(unsigned int)leftDigits
			 right:(unsigned int)rightDigits;
- (void)setFloatValue:(float)aFloat;
- (void)setFont:(NSFont *)fontObject;
- (void)setImage:(NSImage *)anImage;
- (void)setIntValue:(int)anInt;
- (void)setOpaque:(BOOL)flag;
- (void)setRepresentedObject:(id)anObject;
- (void)setScrollable:(BOOL)flag;
- (void)setSelectable:(BOOL)flag;
- (void)setState:(int)value;
- (void)setStringValue:(NSString *)aString;
- (void)setTag:(int)anInt;
- (void)setTarget:(id)anObject;
- (void)setType:(NSCellType)aType;
- (void)setWraps:(BOOL)flag;
- (int)state;
- (NSString *)stringValue;
- (int)tag;
- (void)takeDoubleValueFrom:(id)sender;
- (void)takeFloatValueFrom:(id)sender;
- (void)takeIntValueFrom:(id)sender;
- (void)takeStringValueFrom:(id)sender;
- (id)target;
- (NSRect)titleRectForBounds:(NSRect)theRect;
- (BOOL)trackMouse:(NSEvent *)theEvent
	    inRect:(NSRect)cellFrame
	    ofView:(NSView *)controlView
      untilMouseUp:(BOOL)flag;
- (NSCellType)type;
- (BOOL)wraps;

@end
#endif
