/*
 * Filename: NSColor.h
 * Project:  Pascal's X Kit
 * Module:   Basic application components
 *
 * Function: A small subset of the OpenStep specification for NSColors
 *
 * Note:     This class is not even totally OpenStep compatible.
 *           It is a simple implementation that will allow me to
 *           test other classes (NSView, NSWindow, etc.).
 *           The class will be rewritten later.
 *
 * Author:   Pascal Forget
 * Date:     January 1996
 *
 * Copyright (C) 1996 Pascal Forget. All Rights Reserved.
 */

#ifndef NSCOLOR_H__
#define NSCOLOR_H__

#include "NSGlobal.h"

@interface NSColor : NSObject
{
    NSString *colorSpaceName;
    NSString *catalogName;
    NSString *colorName;
    float alphaComponent;
    float brightnessComponent;
    float redComponent;
    float greenComponent;
    float blueComponent;
    float whiteComponent;
    float cyanComponent;
    float magentaComponent;
    float yellowComponent;
    float blackComponent;
    float saturationComponent;
    float hueComponent;        /* Synonym:  tint, tone */
}

/*
 * +colorWithCalibratedWhite: creates and returns a new NSColor
 * whose color space is NSCalibratedWhiteColorSpace, whose opacity is alpha,
 * and whose grayscale value is 'white'. All values are legal, but values
 * less than 0.0 are set to 0.0, and values greater than 1.0 are set to 1.0.
 */

+ (NSColor *)blackColor;
+ (NSColor *)blueColor;
+ (NSColor *)brownColor;
+ (NSColor *)clearColor;

+ (NSColor *)colorWithCalibratedRed:(float)red
			      green:(float)green 
                               blue:(float)blue
			      alpha:(float)alpha;

+ (NSColor *)colorWithCalibratedWhite:(float)white alpha:(float)alpha;
+ (NSColor *)cyanColor;
+ (NSColor *)darkGrayColor;
+ (NSColor *)grayColor;
+ (NSColor *)greenColor;
+ (BOOL)ignoresAlpha;
+ (NSColor *)lightGrayColor;
+ (NSColor *)magentaColor;
+ (NSColor *)orangeColor;
+ (NSColor *)purpleColor;
+ (NSColor *)redColor;
+ (void)setIgnoresAlpha:(BOOL)flag;
+ (NSColor *)whiteColor;
+ (NSColor *)yellowColor;

- (float)alphaComponent;
- (float)blueComponent;
- (NSString *)colorSpaceName;

- (void)getRed:(float *)red
	 green:(float *)green 
          blue:(float *)blue 
	 alpha:(float *)alpha;

- (void)getWhite:(float *)white 
	 alpha:(float *)alpha;

- (float)greenComponent;
- (float)redComponent;
- (void)set;
- (float)whiteComponent;

@end
#endif
