/*
 * Filename: NSControl.h
 * Project:  Pascal's X Kit
 * Module:   Basic application components
 *
 * Function: This is the Control class.  A control is normally something
 *           that responds to mouse clicks by sending an action to its
 *           target.  Text fields, buttons and scrollers are controls.
 *
 * Author:   Pascal Forget
 * Date:     March 1996
 *
 * Copyright (C) 1996 Free Software Foundation.
 */

#ifndef NSCONTROL_H__
#define NSCONTROL_H__

#include "NSView.h"
#include "NSGlobal.h"
#include "NSFont.h"
#include "NSCell.h"

@interface NSControl : NSView
{
    id target;
    SEL action;
    BOOL ignoresMultiClick;
    BOOL isContinuous;
    BOOL isEnabled;
    int state;
    NSCell *cell;
}

+ (Class)cellClass;
- (SEL)action;
- (NSTextAlignment)alignment;
- (id)cell;
- (double)doubleValue;
- (float)floatValue;
- (NSFont *)font;
- (BOOL)ignoresMultiClick;
- (id)initWithFrame:(NSRect)frameRect;
- (int)intValue;
- (BOOL)isContinuous;
- (BOOL)isEnabled;
- (void)mouseDown:(NSEvent *)theEvent;
- (id)selectedCell;
- (int)selectedTag;
- sendAction;
- (void)setAction:(SEL)aSelector;
- (void)setCell:(NSCell *)aCell;
- (void)setContinuous:(BOOL)flag;
- (void)setDoubleValue:(double)aDouble;
- (void)setEnabled:(BOOL)flag;
- (void)setFloatValue:(float)aFloat;
- (void)setFloatingPointFormat:(BOOL)autoRange
			  left:(unsigned int)leftDigits
			 right:(unsigned int)rightDigits;
- (void)setFont:(NSFont *)fontObject;
- (void)setIntValue:(int)anInt;
- setState:(int)newState;
- (void)setStringValue:(NSString *)str;
- (void)setTag:(int)anInt;
- (void)setTarget:(id)anObject;
- (int)state;
- (NSString *)stringValue;
- (int)tag;
- (void)takeDoubleValueFrom:(id)sender;
- (void)takeFloatValueFrom:(id)sender;
- (void)takeIntValueFrom:(id)sender;
- (void)takeStringValueFrom:(id)sender;
- (id)target;
- (int)toggleState; /* returns the new state */
- (void)updateCell:(NSCell *)aCell;
- (void)updateCellInside:(NSCell *)aCell;
@end
#endif
