/*
 * Filename: NSEvent.h
 * Project:  Pascal's X Kit
 * Module:   Basic components
 *
 * Function: a subset of the OpenStep's NSEvent specification
 *
 * Author:   Pascal Forget
 * Date:     January 1996
 *
 * Copyright (C) 1996 Pascal Forget. All Rights Reserved
 */

#ifndef NSEVENT_H__
#define NSEVENT_H__

#include "NSGlobal.h"

/* Don't #include NSWindow.h here. This would create a circular reference. */

@class NSWindow;

/* Fake class */
@interface NSDPSContext : NSObject
{
}
@end

@interface NSEvent : NSObject
{
    NSString *characters;
    NSString *charactersIgnoringModifiers;
    int clickCount;     /* single-click, double-click, etc. */
    NSDPSContext *_context;
    int _data1;
    int _data2;
    int eventNumber;
    NSEventType eventType;
    unsigned int _flags;
    BOOL isARepeat;
    unsigned short keyCode;
    NSPoint _location;
    float pressure;
    short subtype;
    NSTimeInterval timestamp;
    int trackingNumber;
    void *_userData;
    int windowNumber;
}

+ (NSEvent *)enterExitEventWithType:(NSEventType)type
			   location:(NSPoint)location
		      modifierFlags:(unsigned int)flags
			  timestamp:(NSTimeInterval)time
		       windowNumber:(int)windowNum
			    context:(NSDPSContext *)context
			eventNumber:(int)eventNum
		     trackingNumber:(int)trackingNum
			   userData:(void *)userData;

+ (NSEvent *)keyEventWithType:(NSEventType)type
		     location:(NSPoint)location
		modifierFlags:(unsigned int)flags
		    timestamp:(NSTimeInterval)time
		 windowNumber:(int)windowNum
		      context:(NSDPSContext *)context
		   characters:(NSString *)keys
  charactersIgnoringModifiers:(NSString *)uKeys
		    isARepeat:(BOOL)repeatKey
		      keyCode:(unsigned short)code;

+ (NSEvent *)mouseEventWithType:(NSEventType)type
		       location:(NSPoint)location
		  modifierFlags:(unsigned int)flags
		      timestamp:(NSTimeInterval)time
		   windowNumber:(int)windowNum
			context:(NSDPSContext *)context
		    eventNumber:(int)eventNum
		     clickCount:(int)trackingNum
		       pressure:(float)pressureValue;

+ (NSEvent *)otherEventWithType:(NSEventType)type
		       location:(NSPoint)location
		  modifierFlags:(unsigned int)flags
		      timestamp:(NSTimeInterval)time
		   windowNumber:(int)windowNum
			context:(NSDPSContext *)context
		    subtype:(short)subType
		     data1:(int)data1
		       data2:(int)data2;

- (NSString *)characters;
- (NSString *)charactersIgnoringModifiers;
- (int)clickCount;
- (NSDPSContext *)context;
- (int)data1;
- (int)data2;
- (int)eventNumber;
- (BOOL)isARepeat;
- (unsigned short)keyCode;
- (NSPoint)locationInWindow;
- (unsigned int)modifierFlags;
- (float)pressure;
- (short)subtype;
- (NSTimeInterval)timestamp;
- (int)trackingNumber;
- (NSEventType)type;
- (void *)userData;
- (NSWindow *)window;
- (int)windowNumber;

@end
#endif
