/*
 * Filename: NSMatrix.h
 * Project:  Pascal's X Kit
 * Module:   Basic application components
 *
 * Function: This is the Matrix class. This class corresponds to NSMatrix
 *           of the OpenStep specification, but it also allows for rows
 *           and columns of different sizes.
 *
 * Author:   Pascal Forget <pascal@wsc.com>
 * Date:     October 1995
 *
 * Modified: March 1996
 * By:       Pascal Forget
 * Changes:  Added target-action mechanism, and replaced PXKLists with NSArrays
 *
 * Copyright (C) 1996 Free Software Foundation, Inc.
 */

#ifndef NSMATRIX_H__
#define NSMATRIX_H__

#include "NSControl.h"

@interface NSMatrix : NSControl
{
    NSMutableArray *rows;  /* A list of rows.  Each row is a list of cells */
    NSMutableArray *colWidths;
    NSMutableArray *rowHeights;
    int numCols;
    int numRows;
    NSCell *cellPrototype;
    NSSize interCell;
    Class cellClass;
    BOOL allowsEmptySelection;
    BOOL selectionByRect;
    BOOL drawsBackground;
    BOOL drawsCellBackground;
    NSMutableArray *selectedCells;
    BOOL autoscroll;
    BOOL scrollable;
    BOOL autosize;
    NSMatrixMode mode;
    SEL doubleAction;
    SEL errorAction;
}

+ (Class)cellClass;
+ (void)setCellClass:(Class)classId;

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent;
- (void)addColumn;
- (void)addColumnWithCells:(NSArray *)cellArray;
- (void)addRow;
- (void)addRowWithCells:(NSArray *)cellArray;
- (BOOL)allowsEmptySelection;
- (BOOL)autosizesCells;
- (id)cellAtRow:(int)row column:(int)column;
- (Class)cellClass;
- (NSRect)cellFrameAtRow:(int)row column:(int)column;
- (id)cells;
- (NSSize)cellSize;
- (id)cellWithTag:(int)anInt;
- (void)deselectAllCells;
- (void)deselectSelectedCell;
- (SEL)doubleAction;
- (BOOL)drawsBackground;
- (BOOL)drawsCellBackground;
- empty;
- (SEL)errorAction;
- (void)getNumberOfRows:(int *)rowCount columns:(int *)columnCount;
- (BOOL)getRow:(int *)row column:(int *)column forPoint:(NSPoint)aPoint;
- (BOOL)getRow:(int *)row column:(int *)column ofCell:(NSCell *)aCell;
- (void)highlightCell:(BOOL)flag atRow:(int)row column:(int)column;

- (id)initWithFrame:(NSRect)frameRect;

- (id)initWithFrame:(NSRect)frameRect
	       mode:(int)aMode
	  cellClass:(Class)classId
       numberOfRows:(int)rowsHigh
    numberOfColumns:(int)colsWide;

- (id)initWithFrame:(NSRect)frameRect
	       mode:(int)aMode
	  prototype:(NSCell *)aCell
       numberOfRows:(int)rowsHigh
    numberOfColumns:(int)colsWide;

- (void)insertColumn:(int)column;
- (void)insertColumn:(int)column withCells:(NSArray *)cellArray;
- (void)insertRow:(int)row;
- (void)insertRow:(int)row withCells:(NSArray *)cellArray;
- (NSSize)intercellSpacing;
- (BOOL)isAutoscroll;
- (BOOL)isSelectionByRect;
- (id)makeCellAtRow:(int)row column:(int)column;
- (NSMatrixMode)mode;
- (void)mouseDown:(NSEvent *)theEvent;
- (id)prototype;
- (void)putCell:(id)newCell atRow:(int)row column:(int)column;
- (void)removeColumn:(int)column;
- (void)removeRow:(int)row;
- (void)selectAll:(id)sender;
- (void)selectCellAtRow:(int)row column:(int)column;
- (BOOL)selectCellWithTag:(int)anInt;
- (NSArray *)selectedCells;
- (int)selectedColumn;
- (int)selectedRow;
- (BOOL)sendAction;
- (BOOL)sendDoubleAction;
- (void)setAllowsEmptySelection:(BOOL)flag;
- (void)setAutoscroll:(BOOL)flag;
- (void)setAutosizesCells:(BOOL)flag;
- (void)setDoubleAction:(SEL)aSelector;
- (void)setDrawsBackground:(BOOL)flag;
- (void)setDrawsCellBackground:(BOOL)flag;
- (void)setErrorAction:(SEL)aSelector;
- (void)setIntercellSpacing:(NSSize)aSize;
- (void)setSelectionByRect:(BOOL)flag;
- (void)setCellClass:(Class)classId;
- (void)setPrototype:(NSCell *)aCell;
- (void)setCellSize:(NSSize)aSize;
- (void)setColumnWidth:(int)width at:(int)column;
- (void)setIntercellSpacing:(NSSize)aSize;
- setRowHeight:(int)h at:(int)row;
- (void)setScrollable:(BOOL)flag;
- (void)setState:(int)value atRow:(int)row column:(int)column;
- (void)sizeToCells;

@end
#endif
