/* 
   NSText.h

   The text object

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Scott Christley <scottc@net-community.com>
   Date: 1996
   Modified by: ANOQ of the Sun <anoq@vip.cybercity.dk>
   Date: November 1997
   
   This file is part of the NSXKit.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#ifndef _NSXKit_H_NSText
#define _NSXKit_H_NSText

#include "NSGlobal.h"
#include "NSView.h"
#include "NSFont.h"
#include <Foundation/NSCoder.h>
#include <Foundation/NSNotification.h>
#include "NSTextStorage.h"

typedef struct _NSTextChunk
{
  short growby;
  int allocated;
  int used;
} NSTextChunk;

typedef struct  _NSTextBlock
{
  struct _NSTextBlock *next;
  struct _NSTextBlock *prior;
  struct _tbFlags
  {
    unsigned int malloced:1;
    unsigned int PAD:15;
  } tbFlags;
  short chars;
  unsigned char *text;
} NSTextBlock;

enum
{
  NSTextBlockSize = 512
};

typedef short NSLineDesc;

typedef struct _NSBreakArray
{
  NSTextChunk chunk;
  NSLineDesc breaks[1];
} NSBreakArray;

typedef struct _NSCharArray
{
  NSTextChunk chunk;
  unsigned char text[1];
} NSCharArray;

typedef unsigned short (*NSCharFilterFunc)
  (unsigned short charCode, int flags, NSStringEncoding theEncoding);

typedef struct _NSFSM
{
  const struct _NSFSM *next;
  short delta;
  short token;
} NSFSM;

typedef struct _NSHeightInfo
{
  float newHeight;
  float oldHeight;
  NSLineDesc lineDesc;
} NSHeightInfo;

typedef struct _NSHeightChange
{
  NSHeightInfo heightInfo;
} NSHeightChange;

typedef struct
{
  unsigned int underline:1;
  unsigned int dummy:1;
  unsigned int subclassWantsRTF:1;
  unsigned int graphic:1;
  unsigned int forcedSymbol:1;
  unsigned int RESERVED:11;
} NSRunFlags;

typedef struct _NSRun
{
  id font;
  int chars;
  void *paraStyle;
  int textRGBColor;
  unsigned char superscript;
  unsigned char subscript;
  id info;
  NSRunFlags rFlags;
} NSRun;

typedef struct _NSRunArray
{
  NSTextChunk chunk;
  NSRun runs[1];
} NSRunArray;

typedef struct _NSWidthArray
{
  NSTextChunk chunk;
  float widths[1];
} NSWidthArray;

typedef struct _NSTextChache
{
  int curPos;
  NSRun *curRun;
  int runFirstPos;
  NSTextBlock *curBlock;
  int blockFirstPos;
} NSTextCache;

typedef struct
{
  unsigned int mustMove:1;
  unsigned int isMoveChar:1;
  unsigned int RESERVED:14;
} NSLayFlags;

typedef struct _NSLay
{
  float x,y;
  short offset,chars;
  id font;
  void *paraStyle;
  NSRun *run;
  NSLayFlags lFlags;
} NSLay;

typedef struct _NSLayArray
{
  NSTextChunk chunk;
  NSLay lays[1];
} NSLayArray;

typedef struct _NSLayInfo
{
  NSRect rect;
  float descent;
  float width;
  float left;
  float right;
  float rightIndent;
  NSLayArray *lays;
  NSWidthArray *widths;
  NSCharArray *chars;
  NSTextCache cache;
  NSRect *textClipRect;
  struct _lFlags
  {
    unsigned int horizCanGrow:1;
    unsigned int vertCanGrow:1;
    unsigned int erase:1;
    unsigned int ping:1;
    unsigned int endsParagraph:1;
    unsigned int resetCache:1;
    unsigned int RESERVED:10;
  } lFlags;
} NSLayInfo;

typedef enum _NSParagraphProperty
{
  NSLeftAlignedParagraph,
  NSRightAlignedParagraph,
  NSCenterAlignedParagraph,
  NSJustificationAlignedParagraph,
  NSFirstIndentParagraph,
  NSIndentParagraph,
  NSAddTabParagraph,
  NSRemoveTabParagraph,
  NSLeftMarginParagraph,
  NSRightMarginParagraph
} NSParagraphProperty;

typedef struct _NSSetPt
{
  int cp;
  int line;
  float x,y;
  int c1st;
  float ht;
} NSSetPt;

typedef struct _NSTabStop
{
  short kind;
  float x;
} NSTabStop;

typedef char *(*NSTextFilterFunc)
  (id self, unsigned char *insertText, int insertLength, int position);
typedef int (*NSTextFunc)(id self,NSLayInfo *layInfo);

//Defined in NSGlobal.h
//typedef enum _NSTextAlignment
//{
//  NSLeftTextAlignment,
//  NSRightTextAlignment,
//  NSCenterTextAligment,
//  NSJustifiedTextAlignment,
//  NSNaturalTextAlignment
//} NSTextAlignment;

typedef struct _NSTextStyle
{
  float indent1st;
  float indent2nd;
  float lintHt;
  float descentLine;
  NSTextAlignment alignment;
  short numTabs;
  NSTabStop tabs;
} NSTextStyle;

enum
{
  NSLeftTab
};

enum
{
  NSBackspaceKey = 8,
  NSCarraiageReturnKey = 13,
  NSDeleteKey = 0x7f,
  NSBacktabKey = 25
};

enum
{
  NSIllegalTextMovement = 0,
  NSReturnTextMovement = 0x10,
  NSTabTextMovement = 0x11,
  NSBackTabTextMovement = 0x12,
  NSLeftTextMovement = 0x13,
  NSRightTextMovement = 0x14,
  NSUpTextMovement = 0x15,
  NSDownTextMovement = 0x16
};

//Break tables
const NSFSM *NSCBreakTable;
int NSCBreakTableSize;
const NSFSM *NSEnglishBreakTable;
int NSEnglishBreakTableSize;
const NSFSM *NSEnglishNoBreakTable;
int NSEnglishNoBreakTableSize;

//Character category Tables
const unsigned char *NSCCharCatTable;
const unsigned char *NSEnglishCharCatTable;

//Click Tables
const NSFSM *NSCClickTable;
int NSCClickTableSize;
const NSFSM *NSEnglishClickTable;
int NSEnglishClickTableSize;

//Smart Cut & Paste Tables
const unsigned char *NSCSmartLeftChars;
const unsigned char *NSCSmartRightChars;
const unsigned char *NSEnglishSmartLeftChars;
const unsigned char *NSEnglishSmartRightChars;

  //Copies the array pc to the array dpc and returns a pointer to the copy
NSTextChunk *NSChunkCopy(NSTextChunk *pc,NSTextChunk *dpc);

  //Increases the array identified by the pointer pc to a size of newUsed bytes
NSTextChunk *NSChunkGrow(NSTextChunk *pc,int newUsed);

  //Allocates initial memory for a structure whose first field is an NSTextChunk structure and
  //subsequent field is a variable-sized array. The amount of memory allocated is equal to 
  //initUsed. If initUsed is 0, growBy bytes are allocated. growBy specifies how much memory
  //should be allocated when the chunk grows
NSTextChunk *NSChunkMalloc(int growBy,int initUsed);

  //Increases the amount of memory available for the array identified by the pointer pc, as
  //determined by the array's NSTextChunk.
NSTextChunk *NSChunkRealloc(NSTextChunk *pc);

  //Like NSChunkCopy, but uses the specified zone of memory
NSTextChunk *NSChunkZoneCopy(NSTextChunk *pc,NSTextChunk *dpc,NSZone *zone);

  //Like NSChunkGrow, but uses the specified zone of memory
NSTextChunk *NSChunkZoneGrow(NSTextChunk *pc,int newUsed,NSZone *zone);

  //Like NSChunkMalloc, but uses the specified zone of memory
NSTextChunk *NSChunkZoneMalloc(int growBy,int initUsed,NSZone *zone);

  //Like NSChunkRealloc, but uses the specified zone of memory
NSTextChunk *NSChunkZoneRealloc(NSTextChunk *pc,NSZone *zone);

  //Determines the placement of characters in a line of text. Refers to the NSText object calling
  //the function, and *layInfo is an NSLayInfo struct. The function returns 1 if a word's length
  //exceeds the width of a line and the NSText's charWrap instance varaible is NO. Otherwise, it
  //returns 0.
int NSScanALine(id self,NSLayInfo *layInfo);

  //Draws a line of text, using the global variables set by NSScanALine(). The return value has
  //no signifigance.
int NSDrawALine(id self,NSLayInfo *layInfo);

@interface NSText : NSView <NSCoding>
{
  // Attributes
  id delegate;
  NSTextStorage *_textStorage;
  NSBreakArray *breakArray;
  unsigned int alignment;
  BOOL drawsBackground;
  BOOL isEditable;
  BOOL isRichText;
  BOOL isSelectable;
  BOOL importsGraphics;
  BOOL usesFontPanel;
  BOOL isHorizontallyResizable;
  BOOL isVerticallyResizable;
  BOOL isRulerVisible;
  BOOL isFieldEditor;
  NSColor *textBackgroundColor;
  NSColor *textColor;
  NSFont *defaultFont;
  NSRange selectedRange;
  NSSize maxSize;
  NSSize minSize;
}

//
// Getting and Setting Contents 
//
- (void)replaceRange:(NSRange)range
	     withRTF:(NSData *)rtfData;
- (void)replaceRange:(NSRange)range
	    withRTFD:(NSData *)rtfdData;
- (NSData *)RTFDFromRange:(NSRange)range;
- (NSData *)RTFFromRange:(NSRange)range;
- (void)setText:(NSString *)string;
- (void)setText:(NSString *)string
	  range:(NSRange)range;
- (NSString *)text;

//
// Managing Global Characteristics
//
- (NSTextAlignment)alignment;
- (BOOL)drawsBackground;
- (BOOL)importsGraphics;
- (BOOL)isEditable;
- (BOOL)isRichText;
- (BOOL)isSelectable;
- (void)setAlignment:(NSTextAlignment)mode;
- (void)setDrawsBackground:(BOOL)flag;
- (void)setEditable:(BOOL)flag;
- (void)setImportsGraphics:(BOOL)flag;
- (void)setRichText:(BOOL)flag;
- (void)setSelectable:(BOOL)flag;

//
// Managing Font and Color
//
- (NSColor *)backgroundColor;
- (void)changeFont:(id)sender;
- (NSFont *)font;
- (void)setBackgroundColor:(NSColor *)color;
- (void)setTextColor:(NSColor *)color
	 range:(NSRange)range;
- (void)setFont:(NSFont *)obj;
- (void)setFont:(NSFont *)font
	range:(NSRange)range;
- (void)setTextColor:(NSColor *)color;
- (void)setUsesFontPanel:(BOOL)flag;
- (NSColor *)textColor;
- (BOOL)usesFontPanel;

//
// Managing the Selection
//
- (NSRange)selectedRange;
- (void)setSelectedRange:(NSRange)range;

//
// Sizing the Frame Rectangle
//
- (BOOL)isHorizontallyResizable;
- (BOOL)isVerticallyResizable;
- (NSSize)maxSize;
- (NSSize)minSize;
- (void)setHorizontallyResizable:(BOOL)flag;
- (void)setMaxSize:(NSSize)newMaxSize;
- (void)setMinSize:(NSSize)newMinSize;
- (void)setVerticallyResizable:(BOOL)flag;
- (void)sizeToFit;

//
// Responding to Editing Commands
//
- (void)alignCenter:(id)sender;
- (void)alignLeft:(id)sender;
- (void)alignRight:(id)sender;
- (void)copy:(id)sender;
- (void)copyFont:(id)sender;
- (void)copyRuler:(id)sender;
- (void)cut:(id)sender;
- (void)delete:(id)sender;
- (void)paste:(id)sender;
- (void)pasteFont:(id)sender;
- (void)pasteRuler:(id)sender;
- (void)selectAll:(id)sender;
- (void)subscript:(id)sender;
- (void)superscript:(id)sender;
- (void)underline:(id)sender;
- (void)unscript:(id)sender;

//
// Managing the Ruler
//
- (BOOL)isRulerVisible;
- (void)toggleRuler:(id)sender;

//
// Spelling
//
- (void)checkSpelling:(id)sender;
- (void)showGuessPanel:(id)sender;

//
// Scrolling
//
- (void)scrollRangeToVisible:(NSRange)range;

//
// Reading and Writing RTFD Files
//
- (BOOL)readRTFDFromFile:(NSString *)path;
- (BOOL)writeRTFDToFile:(NSString *)path
	     atomically:(BOOL)flag;

//
// Managing the Field Editor
//
- (BOOL)isFieldEditor;
- (void)setFieldEditor:(BOOL)flag;

//
// Managing the Delegate
//
- (id)delegate;
- (void)setDelegate:(id)anObject;

//
// Implemented by the Delegate
//
- (void)textDidBeginEditing:(NSNotification *)aNotification;
- (void)textDidChange:(NSNotification *)aNotification;
- (void)textDidEndEditing:(NSNotification *)aNotification;
- (BOOL)textShouldBeginEditing:(NSText *)textObject;
- (BOOL)textShouldEndEditing:(NSText *)textObject;

//
// NSCoding protocol
//
- (void)encodeWithCoder:aCoder;
- initWithCoder:aDecoder;

@end

#endif // _NSXKit_H_NSText
