/* 
   NSTextAttachment.h

   Headerfile for textattachment

   Copyright (C) 1997 Free Software Foundation, Inc.

   Written by: ANOQ of the Sun <anoq@vip.cybercity.dk>
   Date: November 1997
   
   This file is part of NSXKit

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _NSXKit_H_NSTextAttachment
#define _NSXKit_H_NSTextAttachment

#include <Foundation/NSCoder.h>
#include "NSView.h"
#include "NSCell.h"
#include "NSFileWrapper.h"

enum
{
  NSAttachmentCharacter = 0xfffc
};

@class NSTextAttachment;

@protocol NSTextAttachmentCell <NSObject>
//FIXME: Shouldn't this protocol conform to NSObject?
//       Or else retain/release might be a problem in NSTextAttachment...

//Drawing 
-(void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)aView;
-(void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView;

//Cell size and position 
- (NSSize)cellSize;
- (NSPoint)cellBaselineOffset;

//Event handling 
- (BOOL)wantsToTrackMouse;
- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)aTextView untilMouseUp:(BOOL)flag;

//Setting the attachment 
-(void)setAttachment:(NSTextAttachment *)anAttachment;
- (NSTextAttachment *)attachment;

@end //NSTextAttachmentCell protocol

@interface NSTextAttachmentCell : NSCell <NSTextAttachmentCell>
{
  NSTextAttachment *_textAttachment;
}

//Methods adopted by NSTextAttachmentCell protocol
- (NSSize)cellSize;
- (NSPoint)cellBaselineOffset;
-(void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)aView;
-(void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView;
- (NSTextAttachment *)attachment;
-(void)setAttachment:(NSTextAttachment *)anAttachment;
- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)aTextView untilMouseUp:(BOOL)flag;
- (BOOL)wantsToTrackMouse;

@end //interface NSTextAttachmentCell

@interface NSTextAttachment : NSObject
{
  NSFileWrapper *fileWrapper;
  id <NSTextAttachmentCell> attachmentCell;
}

//Creating an NSTextAttachment
- (id)initWithFileWrapper:(NSFileWrapper *)aWrapper;

//Setting the file wrapper
- (void)setFileWrapper:(NSFileWrapper *)aWrapper;
- (NSFileWrapper *)fileWrapper;

//Setting the attachment cell
- (void)setAttachmentCell:(id <NSTextAttachmentCell>)aCell;
- (id <NSTextAttachmentCell>)attachmentCell;

@end //interface NSTextAttachment

#endif //_NSXKit_H_NSTextAttachment