/* 
   NSTextStorage.h

   Headerfile for text-storage

   Copyright (C) 1997 Free Software Foundation, Inc.

   Written by: ANOQ of the Sun <anoq@vip.cybercity.dk>
   Date: November 1997
   
   This file is part of NSXKit

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _NSXKit_H_NSTextStorage
#define _NSXKit_H_NSTextStorage

#include "NSAttributedString.h"
#include "NSLayoutManager.h"

@interface NSTextStorage : NSMutableAttributedString
{
  NSMutableArray *_layoutManagers;
  int _changeInLength,_editNest;
  NSRange _editedRange;
  id _delegate;
  unsigned _editedMask;
}

enum
{
  NSTextStorageEditedAttributes = 1,
  NSTextStorageEditedCharacters = 2
};

-(void)addLayoutManager:(NSLayoutManager *)aLayoutManager;
-(void)removeLayoutManager:(NSLayoutManager *)aLayoutManager;
- (NSArray *)layoutManagers;

- (int)changeInLength;
- (void)edited:(unsigned)mask range:(NSRange)oldRange changeInLength:(int)lengthChange;
- (unsigned int)editedMask;
- (NSRange)editedRange;

-(void)endEditing;
-(void)processEditing;

-(void)setDelegate:(id)anObject;
-(id)delegate;

@end//interface NSTextStorage

#endif//_NSXKit_H_NSTextStorage

/*
Methods Implemented By the Delegate

 textStorageDidProcessEditing:

-(void)textStorageDidProcessEditing:(NSNotification *)aNotification

Informs the delegate that an NSTextStorage object has finished processing edits.
The text storage object is available by sending object to aNotification, which is
always an NSTextStorageDidProcessEditingNotification. The delegate can use this
notification to verify the final state of the text storage object; it can't
change the text storage object's characters without leaving it in an inconsistent
state, but if necessary it can change attributes. Note that even in this case
it's possible to put a text storage object into an inconsistent statefor
example by changing the font of a range to one that doesn't support the
characters in that range (such as using a Latin font for Kanji text).



6 Classes: NSTextStorage

 textStorageWillProcessEditing:

-(void)textStorageWillProcessEditing:(NSNotification *)aNotification

Informs the delegate that an NSTextStorage object is about to process edits.
The text storage object is available by sending object to aNotification, which
is always an NSTextStorageWillProcessEditingNotification. The delegate can use
this notification to verify the changed state of the text storage object, and to
make changes to the text storage object's characters or attributes to enforce
whatever constraints it establishes (which doesn't result in this message being
sent again, however). For example, a code editor application might add a
delegate that checks after edits to make sure that all programming language
keywords are set in boldface.

Notifications

NSTextStorageDidProcessEditingNotification

Posted after the NSTextStorage finishes processing edits in processEditing.
Observers other than the delegate shouldn't make further changes to the
NSTextStorage. The notification contains:

Notification Object The NSTextStorage that processed edits.

Userinfo None

NSTextStorageWillProcessEditingNotification

Posted before the NSTextStorage finishes processing edits in processEditing.
Observers other than the delegate shouldn't make further changes to the
NSTextStorage. The notification contains:

Notification Object The NSTextStorage about to process edits.

Userinfo None
*/