/*
 * Filename: NSView.h
 * Project:  Pascal's X Kit
 * Module:   Basic application components
 *
 * Function: This is the NSView class.  As OpenStep compliant as possible.
 *           A NSView uses an X-Windows window.
 *
 * Author:   Pascal Forget
 * Date:     February 1997
 *
 * Copyright (C) 1997 Pascal Forget
 */

#ifndef NSVIEW_H__
#define NSVIEW_H__

#include "NSGlobal.h"
#include "PXKList.h"
#include "NSColor.h"
//#include "NSImage.h"
#include "NSWindow.h"

PXKList *instances;

@interface NSView : NSResponder
{
    int viewNum;
    NSRect frame;
    NSColor *backgroundColor;
    float frameRotation;
    NSView *superview;
    NSMutableArray *subviews;
    NSWindow *window;
    NSMutableArray *cursorRects; /* Should be a hash table */
    int tag;

    /* flags */
    BOOL isOpaque;
    BOOL needsDisplay;
    BOOL notifiesAncestors;
    BOOL autoresizesSubviews;
}

/* GNUstep methods */
+ (NSView *)focusView;
- (void)addSubview:(NSView *)aView;
- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent;
- (NSView *)ancestorSharedWithView:(NSView *)aView;
- (BOOL)autoresizesSubviews;
- (BOOL)canDraw;
- (NSPoint)convertPoint:(NSPoint)aPoint fromView:(NSView *)aView;
- (NSPoint)convertPoint:(NSPoint)aPoint toView:(NSView *)aView;
- (void)display;
- (void)displayIfNeeded;
- (void)displayIfNeededIgnoringOpacity;
- (void)displayRect:(NSRect)aRect;       /* Stupid but useful implementation */
- (void)displayRectIgnoringOpacity:(NSRect)aRect;
- (void)drawRect:(NSRect)rect;
- (void)fax:(id)sender;
- (float)frameRotation;
- (NSRect)frame;
- (NSView *)hitTest:(NSPoint)aPoint;
- (id)initWithFrame:(NSRect)frameRect;
- (BOOL)isDescendantOf:(NSView *)aView;
- (BOOL)isFlipped; // returns NO; subclasses override as needed.
- (BOOL)isOpaque;
- (BOOL)isRotatedFromBase; // returns NO; frame rotation is not supported.
- (void)lockFocus;
- (BOOL)needsDisplay;
- (NSView *)opaqueAncestor;
- (BOOL)postsFrameChangedNotifications;
- (void)print:(id)sender;
- (void)removeFromSuperview;
- (void)replaceSubview:(NSView *)oldView with:(NSView *)newView;
- (void)rotateByAngle:(float)angle;
- (void)setAutoresizesSubviews:(BOOL)flag;
- (void)setFrame:(NSRect)frameRect;
- (void)setFrameOrigin:(NSPoint)newOrigin;
- (void)setFrameRotation:(float)angle;
- (void)setFrameSize:(NSSize)newSize;
- (void)setNeedsDisplay:(BOOL)flag;
- (void)setNeedsDisplayInRect:(NSRect)invalidRect; /* dumb implementation */
- (void)setPostsFrameChangedNotifications:(BOOL)flag;
- (void)setTag:(int)anInt;
- (void)sortSubviewsUsingFunction:(int (*)(id,id,void *))compare
			  context:(void *)context;
- (NSArray *)subviews;
- (NSView *)superview;
- (void)superviewSizeChanged:(NSSize)oldSize;
- (int)tag;
- (void)unlockFocus;
- (void)viewWillMoveToWindow:(NSWindow *)newWindow;
- (id)viewWithTag:(int)aTag;
- (NSWindow *)window;
- (NSRect)bounds;

@end

@interface NSView (NonGNUStep)
- (NSColor *)backgroundColor;
- (void)resizeSubviewsOldSize:(NSSize)oldSize newSize:(NSSize)newSize;
- (void)resizeSubviewsOldSize:(NSSize)oldSize;
- (void)setOpaque:(BOOL)flag;
- (void)setSuperview:(NSView *)aView;
- (int)viewNum; 
@end

#endif
