/*
 * Filename: NSWindow.h
 * Project:  Pascal's OpenStep Application Kit for X Windows
 * Module:   Basic application components
 *
 * Function: This is the NSWindow class
 *
 * Author:   Pascal Forget <pascal@wsc.com>
 * Date:     February 1996
 *
 * Copyright (C) 1996 Free Software Foundation, Inc.
 *
 * This file is part of the GNU OpenStep UI kit Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef NSWINDOW_H__
#define NSWINDOW_H__

#include "NSGlobal.h"
#include "NSResponder.h"
#include "NSColor.h"

@class NSView;

//typedef enum _NSBackingStoreType {
//    NSBufferedBackingStore
//} NSBackingStoreType;

@interface NSWindow : NSResponder
{
    NSColor *backgroundColor;
    NSBackingStoreType backingType;
    NSRect _contentRect;
    NSView *contentView;
    BOOL deferred;
    id delegate;
    NSRect frame;
    NSView *frameView;
    BOOL isDocumentEdited;
    BOOL isFlushWindowDisabled;
    BOOL isVisible;
    NSString *representedFilename;
    unsigned int styleMask;
    NSString *title;
    int windowNum;
}

- (NSColor *)backgroundColor;
- (NSBackingStoreType)backingType;
- (void)becomeKeyWindow;
- (void)becomeMainWindow;
- (BOOL)canBecomeKeyWindow;
- (BOOL)canBecomeMainWindow;
- (void)close;
- (id)contentView;
- (id)delegate;
- (void)disableFlushWindow;
- (void)display;
- (void)enableFlushWindow;
- (NSRect)frame;
- (id)initWithContentRect:(NSRect)contentRect
		styleMask:(unsigned int)aStyle
		  backing:(NSBackingStoreType)bufferingType
		    defer:(BOOL)flag;

/* Class NSScreen not implemented yet */

#if 0
- (id)initWithContentRect:(NSRect)contentRect
		styleMask:(unsigned int)aStyle
		  backing:(NSBackingStoreType)bufferingType
		    defer:(BOOL)flag
		   screen:(NSScreen *)aScreen;
#endif

- (BOOL)makeFirstResponder:(NSResponder *)aResponder;
- (BOOL)isDocumentEdited;
- (BOOL)isFlushWindowDisabled;
- (BOOL)isKeyWindow; /* Stub implementation - always returns YES */
- (BOOL)isVisible;
- (void)mouseDown:(NSEvent *)theEvent;
- (void)mouseUp:(NSEvent *)theEvent;
- (NSString *)representedFilename;
- (void)rightMouseDown:(NSEvent *)theEvent;
- (void)setBackgroundColor:(NSColor *)color;
- (void)setBackingType:(NSBackingStoreType)type;
- (void)setContentSize:(NSSize)aSize;
- (void)setContentView:(NSView *)aView;
- (void)setDelegate:(id)anObject;
- (void)setDocumentEdited:(BOOL)flag;
- (void)setFrame:(NSRect)frameRect display:(BOOL)flag;
- (void)setFrameOrigin:(NSPoint)aPoint;
- (void)setFrameTopLeftPoint:(NSPoint)aPoint;
- (void)setRepresentedFilename:(NSString *)aString;
- (void)setTitle:(NSString *)aString;
- (void)setTitleWithRepresentedFilename:(NSString *)aString;
- (unsigned int)styleMask;
- (NSString *)title;
- (int)windowNumber;
@end
#endif

