/*
 * Filename: PXKDrawing.h
 * Project:  Pascal's AppKit for X Windows
 * Module:   Drawing Engine
 *
 * Function: Basic drawing functions of the kit
 *
 * Author:   Pascal Forget <pascal@wsc.com>
 * Date:     January 1997
 *
 * Copyright (C) 1997 Pascal Forget.  All Rights Reserved.
 */

#ifndef PXKDRAWING_H__
#define PXKDRAWING_H__

#include <Foundation/NSData.h>
#include "PXKList.h"
#include "NSGlobal.h"
#include "NSColor.h"
#include "NSView.h"
#include "NSFont.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <X11/cursorfont.h>
#include <math.h>
#include <objc/Object.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

//FIXME:
//------
//In the NSBitmapImageRep class I allocate 32 bits per pixel for the
//internal storage format when the display is running 32bits per
//pixel. I do it to avoid an image-conversion from 24bit to 32bit
//when sending the image to the X-server. I don't know if this
//violates the OpenStep-specification. I haven't found any other
//ways to avoid this image-conversion, and the image-conversion
//takes a lot of time. The image-conversion has been written
//though and is in PXKDrawing.m in the function NSDrawBitmap().

/* Rectangle Drawing Functions */

void NSEraseRect(NSRect aRect);          /* fills the rect with white.       */
void NSHighlightRect(NSRect aRect);      /* LTGRAY->WHITE, and vice-versa.   */
void NSRectClip(NSRect aRect);           /* Intersects the current clip path */
void NSRectClipList(const NSRect *rects, int count);
void NSRectFill(NSRect aRect);           /* Fills rect using current color   */
void NSRectFillList(const NSRect *rects, int count);

/*
 * NSRectFillWithGrays() Fills each rectangle in the array 'rects'
 * with the gray whose value is stored at the corresponding location
 * in the array 'grays'.  Both arrays must be count elements long.
 * Avoid overlapping rectangles, because the drawing order can't be
 * guaranteed.
 */

void NSRectFillListWithGrays(const NSRect *rects,const float *grays,int count);


/* Private function */
NSPoint intersectionPoint(NSRect lineA, NSRect lineB);
float dist2D(NSPoint a, NSPoint b);
NSRect clipLineToRect(NSRect aLine, NSRect aRect);
void setCurrentColor(NSColor *aColor);

/* Draw a Bordered Rectangle */

void NSDrawButton(NSRect aRect, NSRect clipRect);    /* Bordered ltgray rect */
void NSDrawGrayBezel(NSRect aRect, NSRect clipRect); /* pushed-in button     */
void NSDrawGroove(NSRect aRect, NSRect clipRect);    /* like a Box           */

#if 0
void NSDrawTiledRects(NSRect boundsRect,
		      NSRect clipRect,
		      const NSRectEdge *sides,
		      const float *grays,
		      int count);
#endif

void NSDrawWhiteBezel(NSRect aRect, NSRect clipRect); /* like a TextField    */
void NSFrameRect(NSRect aRect); /* Draws a frame of width 1.0 use curr color */
void NSFrameRectWithWidth(NSRect aRect, float frameWidth);
void NSDrawBitmap(NSRect rect,int pixelsWide,int pixelsHigh,
       int bitsPerSample,int samplesPerPixel,int bitsPerPixel,
       int bytesPerRow,BOOL isPlanar,BOOL hasAlpha,
       NSString *colorSpaceName,const unsigned char *const data[5]);

/* Private functions */
void pxkDrawString(const char *string, NSPoint location);
int pxkWidthOfString(NSString *string, NSFont *aFont);

void pxkSetCurrentPos(NSPoint newPosition);
NSPoint pxkGetCurrentPos(void);
unsigned int pxkFVRBitsPerSample(void);
unsigned int pxkFVRSamplesPerPixel(void);
BOOL pxkFVRHasAlpha(void);
BOOL pxkFVRIsPlanar(void);
NSMutableData *pxkFVRGrab(NSRect rect);
BOOL pxkCopyWindowRect(NSWindow *win,NSRect rect);

#endif
