/*
 * Filename: PXKGeometry.h
 * Project:  Pascal's OpenStep AppKit for X Windows
 * Module:   Drawing Engine
 *
 * Function: A list of related functions for creating and destroying
 *           windows and views, resizing and moving them.
 *
 * Author:   Pascal Forget <pascal@wsc.com>
 * Date:     Januray 1997
 *
 * Copyright (C) 1997 Pascal Forget.  All Rights Reserved.
 */

#ifndef PXKGEOMETRY_H__
#define PXKGEOMETRY_H__

#include "PXKList.h"
#include "NSGlobal.h"
#include "NSColor.h"
#include "NSView.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <X11/cursorfont.h>
#include <math.h>
#include <objc/Object.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

static Display *pxkDisplay;

/*
 * initializeDrawingSystem() is called once from the NSApplication object
 * when the application starts
 */

int initializeDrawingEngine(void);

/* Creating and destroying views and windows */

int createView(NSView *sender, int superviewNum);
int createWindow(NSWindow *sender);
int destroyView(int viewNum);
int destroyWindow(int windowNum);

void displayView(int viewNum);
void displayWindow(int windowNum);
void drawViewBackground(int viewNum);

void setViewFrame(int viewNum, NSRect frameRect);
void setViewFrameOrigin(int viewNum, NSPoint newOrigin);
void setViewFrameRotation(int viewNum, float angle);
void setViewFrameSize(int viewNum, NSSize newSize);

void setWindowFrame(int windowNum, NSRect frameRect);
void setWindowFrameOrigin(int windowNum, NSPoint newOrigin);
void setWindowFrameSize(int windowNum, NSSize newSize);
void setWindowTitle(int windowNum, NSString *title);

/* Utility functions not part of public API */

void closeWindow(int windowNum);
void setCurrentColor(NSColor *aColor);
void load_font(XFontStruct **font_info);
NSPoint pxkLocationInWindow(Window xWindow, int x, int y);

/* Functions to be called by other elements of the drawing engine */

int indexOfViewNumber(int viewNum);
int indexOfWindowNumber(int windowNum);
int indexOfXWindowRepresentingAView(Window xWindow);
int indexOfXWindowRepresentingAWindow(Window xWindow);

NSWindow *windowForXWindow(Window xWindow);
int numberForXWindow(Window win);

/* These are called from NSApplication */

void miniaturizeAll(id sender);
void preventWindowOrdering(void);
void updateWindows(void);

Display *
xDisplay(void); 

#endif
