/*
 * Filename: NSActionCell.m
 * Project:  Pascal's X Kit
 * Module:   Basic components
 *
 * Function: See header file
 *
 * Author:   Pascal Forget
 * Date:     March 1996
 *
 * Copyright (C) 1996 Free Software Foundation, Inc.
 */

#include "NSActionCell.h"
#include "NSControl.h"

@implementation NSActionCell

- init;
{
    [super init];
    _controlView = nil;
    return self;
}

- (SEL)action;
{
    return action;
}

- (NSView *)controlView;
{
    return _controlView;
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView;
{
    _controlView = controlView;
    [super drawInteriorWithFrame:cellFrame inView:controlView];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView;
{
    _controlView = controlView;
    [self drawInteriorWithFrame:cellFrame inView:controlView];
}

- (void)setAction:(SEL)aSelector;
{
    action = aSelector;
}

- (void)setAlignment:(NSTextAlignment)mode;
{
    [super setAlignment:mode];

    if ((_controlView != nil) 
	&& ([_controlView isKindOfClass:[NSControl class]]))
    {
	[_controlView updateCell:self];
    }
}

- (void)setBezeled:(BOOL)flag;
{
    [super setBezeled:flag];

    if ((_controlView != nil) &&
	([_controlView isKindOfClass:[NSControl class]]))
    {
	[_controlView updateCell:self];
    }
}

- (void)setBordered:(BOOL)flag;
{
    [super setBordered:flag];

    if ((_controlView != nil)
	&& ([_controlView isKindOfClass:[NSControl class]]))
    {
	[_controlView updateCell:self];
    }    
}

- (void)setEnabled:(BOOL)flag;
{
    [super setEnabled:flag];

    if ((_controlView != nil) &&
	([_controlView isKindOfClass:[NSControl class]]))
    {
	[_controlView updateCell:self];
    }    
}

- (void)setFloatingPointFormat:(BOOL)autoRange
			  left:(unsigned int)leftDigits
			 right:(unsigned int)rightDigits;
{
    [super setFloatingPointFormat:autoRange
	   left:leftDigits
	   right:rightDigits];

    if ((_controlView != nil)
	&& ([_controlView isKindOfClass:[NSControl class]]))
    {
	[_controlView updateCell:self];
    }     
}

- (void)setStringValue:(NSString *)aString;
{
    [super setStringValue:aString];
    
    if ((_controlView != nil)
	&& ([_controlView isKindOfClass:[NSControl class]]))
    {
	[_controlView updateCell:self];
    }    
}

- (void)setTag:(int)anInt;
{
    tag = anInt;
}

- (void)setTarget:(id)anObject;
{
    target = anObject;
}

/*
 * Although the OpenStep Spec says that the stringValue method is
 * overridden by NSActionCell, I don't know what this implementation
 * should do differently than NSCell's implementation.  Therefore,
 * I simply call [super stringValue].
 */
- (NSString *)stringValue;
{
    return [super stringValue];
}

- (int)tag;
{
    return tag;
}

- (id)target;
{
    return target;
}

@end
