/*
 * Filename: NSBox.m
 * Project:  Pascal's X kit
 * Module:   Basic application components
 *
 * Function: This is the Box class
 *
 * Author:   Pascal Forget
 * Date:     December 1996
 *
 * Copyright (C) 1996 Pascal Forget. All Rights Reserved.
 */

#include "NSBox.h"
#include "NSCell.h"
#include "PXKDrawing.h"
#include "PXKGeometry.h"

@interface NSBox (Private)

- (void)drawBorder;
- (void)drawTitle;
- (void)updateBoxRect;
- (void)updateTitleRect;
- (float)widthOfCell;
@end

@implementation NSBox (Private)

- (void)drawBorder;
{
    NSRect rect = [self frame]; // frame is an NSView's instance variable
    NSRect clipRect; 
    
    rect.origin.x = 0.0;
    rect.origin.y = 0.0;
    
    clipRect = rect;
    
    rect.origin.x = 1.0;    
    rect.size.width -= 2.0;
    rect.size.height -= 1.0;
    
    if ((titleCell != nil) && (titlePosition == NSAtTop))
    {
	rect.origin.y = 5.0;
	rect.size.height -= 5.0;
    }
    
    NSDrawGroove(rect, clipRect);
}

- (void)drawTitle;
{
    NSRect rect = {{0.0, 0.0}, {80.0, 16.0}};
    
    rect.size.width = [self widthOfCell] + 2.0;

    // Center the title horizontally
    rect.origin.x = (frame.size.width - rect.size.width) / 2.0;
    
    if ((titleCell != nil) && (titlePosition == NSAtTop))
    {
	[titleCell drawWithFrame: rect inView: self];
    }
}

- (void)updateBoxRect;
{

}

- (void)updateTitleRect;
{

}

- (float)widthOfCell;
{
    return [[titleCell font] widthOfString: [titleCell stringValue]];
}

@end

@implementation NSBox

- (NSRect)borderRect;
{
    return borderRect;
}

- (NSBorderType)borderType;
{
    return borderType;
}

- (void)display;
{
    NSRect rect = [self frame];

    rect.origin.x = 0.0;
    rect.origin.y = 0.0;
    
    [super display];
    
    if ([[self subviews] count] == 0)
    {
	[self addSubview: contentView];
    }
    
    [[NSColor lightGrayColor] set];

    [(NSView *)self lockFocus];    
    NSRectFill(rect);
    [self drawBorder];
    [self drawTitle];
    [(NSView *)self unlockFocus];
    [contentView display];
}

- (id)initWithFrame:(NSRect)frameRect;
{
    [super initWithFrame: frameRect];

    contentViewMargins.height = 0;
    contentViewMargins.width = 0;

//    NSBorderType borderType;
    title = nil;
    titleCell = [[NSCell alloc] initTextCell: @"cell"];
    titlePosition = NSAtTop;
    
    contentView = [[NSView alloc] initWithFrame:frameRect];
    
    return self;
}

- (void)setBorderType:(NSBorderType)aType;
{
    borderType = aType;
}

- (void)setTitle:(NSString *)aString;
{
    if (title != nil) {
	[title release];
	title = nil;
    }

    if (aString != nil) {
	title = [aString retain];
    }
}

- (void)setTitleFont:(NSFont *)fontObj;
{
    if (titleFont != nil) {
	[titleFont release];
	titleFont = nil;
    }

    if (fontObj != nil) {
	titleFont = [fontObj retain];
    }
}

- (void)setTitlePosition:(NSTitlePosition)aPosition;
{
    titlePosition = aPosition;
}

- (NSString *)title;
{
    return title;
}

- (id)titleCell;
{
    return titleCell;
}

- (NSFont *)titleFont;
{
    return titleFont;
}

- (NSTitlePosition)titlePosition;
{
    return titlePosition;
}

- (NSRect)titleRect;
{
    return titleRect;
}

- (id)contentView;
{
    return [subviews firstObject];
}

- (NSSize)contentViewMargins;
{
    return contentViewMargins;
}

- (void)setContentView:(NSView *)aView;
{
    contentView =  aView;
    
    if ([subviews count] > 0) {
	[subviews replaceObjectAtIndex: 0 with: aView];
    } else {
	[subviews addObject: aView];
    }
}
    
- (void)setContentViewMargins:(NSSize)offsetSize;
{
    contentViewMargins.height = offsetSize.height;
    contentViewMargins.width = offsetSize.width;
}

- (void)setFrameFromContentFrame:(NSRect)contentFrame;
{
    frame.size.height = contentViewMargins.height * 2 + 
	contentFrame.size.height;
    frame.size.width = contentViewMargins.width * 2 + 
	contentFrame.size.width;
}

- (void)sizeToFit;
{
    NSRect contentFrame = [[subviews firstObject] frame];

    frame.size.height = contentViewMargins.height * 2 + 
	contentFrame.size.height;
    frame.size.width = contentViewMargins.width * 2 + 
	contentFrame.size.width;
}

@end
