/*
 * Filename: NSEvent.m
 * Project:  Pascal's X Kit
 * Module:   Basic components
 *
 * Function: See header file for a description
 *
 * Author:   Pascal Forget
 * Date:     January 1996
 *
 * Copyright (C) 1996 Pascal Forget.  All Rights Reserved
 */

#include "NSEvent.h"
#include "NSWindow.h"
#include "NSApplication.h"

@interface NSEvent (Private)

- (NSEvent *)initGenericEventWithType:(NSEventType)type
			     location:(NSPoint)location
			modifierFlags:(unsigned int)flags
			    timestamp:(NSTimeInterval)time
			 windowNumber:(int)windowNum
			      context:(NSDPSContext *)context
			  eventNumber:(int)eventNum
		       trackingNumber:(int)trackingNum
			     userData:(void *)userData
			   characters:(NSString *)keys
	  charactersIgnoringModifiers:(NSString *)uKeys
			    isARepeat:(BOOL)repeatKey
			      keyCode:(unsigned short)code   
			   clickCount:(int)clickNum
			     pressure:(float)pressureValue
			      subtype:(short)subType
				data1:(int)data1
				data2:(int)data2
			    methodId:(unsigned short)identifier;
@end

@implementation NSEvent (Private)

- (NSEvent *)initGenericEventWithType:(NSEventType)type
			     location:(NSPoint)location
			modifierFlags:(unsigned int)flags
			    timestamp:(NSTimeInterval)time
			 windowNumber:(int)windowNum
			      context:(NSDPSContext *)context
			  eventNumber:(int)eventNum
		       trackingNumber:(int)trackingNum
			     userData:(void *)userData
			   characters:(NSString *)keys
	  charactersIgnoringModifiers:(NSString *)uKeys
			    isARepeat:(BOOL)repeatKey
			      keyCode:(unsigned short)code   
			   clickCount:(int)clickNum
			     pressure:(float)pressureValue
			      subtype:(short)subType
				data1:(int)data1
				data2:(int)data2
			    methodId:(unsigned short)identifier;
{
    [self init];

    eventType = type;
    _location = location;
    _flags = flags;
    timestamp = time;
    windowNumber = windowNum;
    _context = context;
    eventNumber = eventNum;
    trackingNumber = trackingNum;
    _userData = userData;
    characters = keys;
    charactersIgnoringModifiers = uKeys;
    isARepeat = repeatKey;
    keyCode = code;
    clickCount = clickNum;
    pressure = pressureValue;
    subtype = subType;
    _data1 = data1;
    _data2 = data2;
    return self;
}

@end

@implementation NSEvent

+ (NSEvent *)enterExitEventWithType:(NSEventType)type
			   location:(NSPoint)location
		      modifierFlags:(unsigned int)flags
			  timestamp:(NSTimeInterval)time
		       windowNumber:(int)windowNum
			    context:(NSDPSContext *)context
			eventNumber:(int)eventNum
		     trackingNumber:(int)trackingNum
			   userData:(void *)userData;
{
    return [[[NSEvent alloc] initGenericEventWithType:type
			     location:location
			     modifierFlags:flags
			     timestamp:time
			     windowNumber:windowNum
			     context:context
			     eventNumber:eventNum
			     trackingNumber:trackingNum
			     userData:userData
			     characters:nil
			     charactersIgnoringModifiers:nil
			     isARepeat:NO
			     keyCode:0
			     clickCount:0
			     pressure:1.0
			     subtype:0
			     data1:0
			     data2:0
			     methodId:0] autorelease];
}

+ (NSEvent *)keyEventWithType:(NSEventType)type
		     location:(NSPoint)location
		modifierFlags:(unsigned int)flags
		    timestamp:(NSTimeInterval)time
		 windowNumber:(int)windowNum
		      context:(NSDPSContext *)context
		   characters:(NSString *)keys
  charactersIgnoringModifiers:(NSString *)uKeys
		    isARepeat:(BOOL)repeatKey
		      keyCode:(unsigned short)code;
{
    return [[[NSEvent alloc] initGenericEventWithType:type
			     location:location
			     modifierFlags:flags
			     timestamp:time
			     windowNumber:windowNum
			     context:context
			     eventNumber:0
			     trackingNumber:0
			     userData:NULL
			     characters:keys
			     charactersIgnoringModifiers:uKeys
			     isARepeat:repeatKey
			     keyCode:code
			     clickCount:0
			     pressure:1.0
			     subtype:0
			     data1:0
			     data2:0
			     methodId:1] autorelease];
}

+ (NSEvent *)mouseEventWithType:(NSEventType)type
		       location:(NSPoint)location
		  modifierFlags:(unsigned int)flags
		      timestamp:(NSTimeInterval)time
		   windowNumber:(int)windowNum
			context:(NSDPSContext *)context
		    eventNumber:(int)eventNum
		     clickCount:(int)clickNum
		       pressure:(float)pressureValue;
{
    return [[[NSEvent alloc] initGenericEventWithType:type
			     location:location
			     modifierFlags:flags
			     timestamp:time
			     windowNumber:windowNum
			     context:context
			     eventNumber:eventNum
			     trackingNumber:0
			     userData:NULL
			     characters:nil
			     charactersIgnoringModifiers:nil
			     isARepeat:NO
			     keyCode:0
			     clickCount:clickNum
			     pressure:pressureValue
			     subtype:0
			     data1:0
			     data2:0
			     methodId:2] autorelease];
}

+ (NSEvent *)otherEventWithType:(NSEventType)type
		       location:(NSPoint)location
		  modifierFlags:(unsigned int)flags
		      timestamp:(NSTimeInterval)time
		   windowNumber:(int)windowNum
			context:(NSDPSContext *)context
			subtype:(short)subType
			  data1:(int)data1
			  data2:(int)data2;
{
    return [[[NSEvent alloc] initGenericEventWithType:type
			     location:location
			     modifierFlags:flags
			     timestamp:time
			     windowNumber:windowNum
			     context:context
			     eventNumber:0
			     trackingNumber:0
			     userData:NULL
			     characters:nil
			     charactersIgnoringModifiers:nil
			     isARepeat:NO
			     keyCode:0
			     clickCount:0
			     pressure:1.0
			     subtype:subType
			     data1:data1
			     data2:data2
			     methodId:3] autorelease];
}

- (NSString *)characters;
{
    return characters;
}

- (NSString *)charactersIgnoringModifiers;
{
    return charactersIgnoringModifiers;
}

- (int)clickCount;
{
    return clickCount;
}

- (NSDPSContext *)context;
{
    return _context;
}

- (int)data1;
{
    return _data1;
}

- (int)data2;
{
    return _data2;
}

- (int)eventNumber;
{
    return eventNumber;
}

- (BOOL)isARepeat;
{
    return isARepeat;
}

- (unsigned short)keyCode;
{
    return keyCode;
}
    
- (NSPoint)locationInWindow;
{
    return _location;
}

- (unsigned int)modifierFlags;
{
    return _flags;
}

- (float)pressure;
{
    return pressure;
}

- (short)subtype;
{
    return subtype;
}

- (NSTimeInterval)timestamp;
{
    return timestamp;
}

- (int)trackingNumber;
{
    return trackingNumber;
}

- (NSEventType)type;
{
    return eventType;
}

- (void *)userData;
{
    return _userData;
}

- (NSWindow *)window;
{
    return [NSApp windowWithWindowNumber:windowNumber];
}

- (int)windowNumber;
{
    return windowNumber;
}

@end

