/*
 * Filename: NSFont.m
 * Project:  Pascal's X kit
 * Module:   Basic application components
 *
 * Function: see header file
 *
 * Author:   Pascal Forget
 * Date:     March 1996
 *
 * Copyright (C) 1996 Free Software Foundation
 */

#include "NSFont.h"
#include "misc.h"

NSFont *_currentFont;

#define SYSTEM_FONT_NAME @"fixed"
#define FIXED_PITCH_FONT @"courier"
#define APP_FONT_NAME @"helvetica"

@interface NSFont (Private)
- (void)setBold:(BOOL)flag;
- (void)setItalic:(BOOL)flag;
- (void)setNarrow:(BOOL)flag;
- (void)setFontName:(NSString *)aString;
- (void)setPointSize:(int)size;
@end

@implementation NSFont

+ (NSFont *)boldSystemFontOfSize:(int)fontSize;
{
    NSFont *newFont = [NSFont fontWithName: SYSTEM_FONT_NAME size: fontSize];
    [newFont setBold: YES];
    return newFont;
}

+ (NSFont *)fontWithName:(NSString *)aString size:(int)fontSize;
{
    NSFont *newFont = [[NSFont alloc] init];
    [newFont setFontName: aString];
    [newFont setPointSize: fontSize];
    return newFont;
}

+ (NSFont *)systemFontOfSize:(int)fontSize;
{
    return [NSFont fontWithName: SYSTEM_FONT_NAME size: fontSize];
}

+ (NSFont *)userFixedPitchFontOfSize:(int)fontSize;
{
    return [NSFont fontWithName: FIXED_PITCH_FONT size: fontSize];
}

+ (NSFont *)userFontOfSize:(int)fontSize;
{
    return [NSFont fontWithName: APP_FONT_NAME size: fontSize];
}

- init;
{
    [super init];
    bold = NO;
    italic = NO;
    narrow = NO;
    return self;
}

- (NSString* )displayName;
{
    return displayName;
}

- (NSString* )familyName;
{
    return familyName;
}

- (NSString *)fontName;
{
    return fontName;
}

- (BOOL)isBaseFont;
{
    return isBaseFont;
}

- (int)pointSize;
{
    return _pointSize;
}

- (NSFont *)printerFont;
{
    return printerFont;
}

- (NSFont *)screenFont;
{
    return screenFont;
}

- (void)set;
{
    _currentFont = self;
}

- (float)widthOfString:(NSString *)string;
{
    /* rough guesstimate.  Should be replaced by a call to XTextWidth() */    
    float guesstimate = 0.0;
    int xlibWidth = pxkWidthOfString(string, self);
    
    if (string != nil) {
	guesstimate = (8.0 * [string length] + (_pointSize / 10.0));
    }
    
//    fprintf(stdout, "Guesstimate string width: %4.2f\n", guesstimate);
    dbug(msg("xlib width of string: %i\n", xlibWidth));
    
    return (float)xlibWidth;
}

- (float *)widths;
{
    return widths;
}

/* Non-standard methods */

+ (NSFont *)currentFont;
{
    return _currentFont;
}

- (BOOL)isBold;
{
    return bold;
}

- (BOOL)isItalic;
{
    return italic;
}

- (BOOL)isNarrow;
{
    return narrow;
}

- (void)setBold:(BOOL)flag;
{
    bold = flag;
}

- (void)setItalic:(BOOL)flag;
{
    italic = flag;
}

- (void)setNarrow:(BOOL)flag;
{
    narrow = flag;
}

- (void)setFontName:(NSString *)aString;
{
    if (familyName != nil) {
	fprintf(stderr, "Cannot change the name of a font.\n");
    } else {
	familyName = [aString retain];
    }
}

- (void)setPointSize:(int)size;
{
    _pointSize = size;
}

@end
