/*
 * Filename: NSResponder.m
 * Project:  Pascal's X Kit
 * Module:   Basic components
 *
 * Function: See header file for a description
 *
 * Author:   Pascal Forget
 * Date:     January 1996
 *
 *
 * Copyright (C) 1996 Pascal Forget.  All Rights Reserved
 */

#include "NSResponder.h"
#include "misc.h"

@implementation NSResponder

+(void)initialize;
{
}

- (BOOL)acceptsFirstResponder;
{
    return NO; /* Subclasses override */
}

- (BOOL)becomeFirstResponder;
{
    if ((isFirstResponder == NO) && ([self acceptsFirstResponder])) {
	isFirstResponder = YES;
	return YES;
    }
    return NO;
}

- (void)flagsChanged:(NSEvent *)theEvent; 
{
    if (nextResponder) {
	[nextResponder flagsChanged:theEvent];
    } else {
	[self noResponderFor:@selector(flagsChanged:)];
    }
}

- (void)helpRequested:(NSEvent *)theEvent; 
{
    if (nextResponder) {
	[nextResponder helpRequested:theEvent];
    } else {
	[self noResponderFor:@selector(helpRequested:)];
    }
}

- init;
{
    [super init];

    isFirstResponder = NO;
    nextResponder = nil;

    return self;
}

- (void)keyDown:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder keyDown:theEvent];
    } else {
	[self noResponderFor:@selector(keyDown:)];
    }
}    

- (void)keyUp:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder keyUp:theEvent];
    } else {
	[self noResponderFor:@selector(keyUp:)];
    }
}    

- (void)mouseDown:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder mouseDown:theEvent];
    } else {
	[self noResponderFor:@selector(mouseDown:)];
    }
}    

- (void)mouseDragged:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder mouseDragged:theEvent];
    } else {
	[self noResponderFor:@selector(mouseDragged:)];
    }
}    

- (void)mouseEntered:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder mouseEntered:theEvent];
    } else {
	[self noResponderFor:@selector(mouseEntered:)];
    }
}    

- (void)mouseExited:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder mouseExited:theEvent];
    } else {
	[self noResponderFor:@selector(mouseExited:)];
    }
}    

- (void)mouseMoved:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder mouseMoved:theEvent];
    } else {
	[self noResponderFor:@selector(mouseMoved:)];
    }
}    

- (void)mouseUp:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder mouseUp:theEvent];
    } else {
	[self noResponderFor:@selector(mouseUp:)];
    }
}    

- (NSResponder *)nextResponder;
{
    return nextResponder;
}

- (void)noResponderFor:(SEL)eventSelector;
{
    if (eventSelector == (SEL)@selector(keyDown:)) {
//	NSBeep();
    }
}

- (BOOL)performKeyEquivalent:(NSEvent *)theEvent;
{
    return NO; /* Subclasses override */
}

- (BOOL)resignFirstResponder;
{
    isFirstResponder = NO;
    return YES;
}

- (void)rightMouseDown:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder rightMouseDown:theEvent];
    } else {
	[self noResponderFor:@selector(rightMouseDown:)];
    }
}    

- (void)rightMouseDragged:(NSEvent *)theEvent; /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder rightMouseDragged:theEvent];
    } else {
	[self noResponderFor:@selector(rightMouseDragged:)];
    }
}    

- (void)rightMouseUp:(NSEvent *)theEvent;  /* subclasses should override */
{
    if (nextResponder) {
	[nextResponder rightMouseUp:theEvent];
    } else {
	[self noResponderFor:@selector(rightMouseUp:)];
    }
}    

- (void)setNextResponder:(NSResponder *)aResponder;
{
    nextResponder = aResponder;
}

- (BOOL)tryToPerform:(SEL)anAction with:(id)anObject;
{
    if ([self respondsToSelector:anAction]) {
	[self perform:anAction withObject:anObject];
	return YES;
    } else if (nextResponder) {
	return [nextResponder tryToPerform:anAction with:anObject];
    } else {
	return NO;
    }
}    

- (id)validRequestorForSendType:(NSString *)typeSent
		     returnType:(NSString *)typeReturned;
{
    if (nextResponder != nil) {
	return [(NSResponder *)nextResponder perform:
				@selector(validRequestForSendType:returnType:)
				withObject:typeSent
				withObject:typeReturned];
    }
    return nil;
}

@end

