/* 
   NSStringDrawing.m

   Implementation of AppKit additions for NSString class

   Copyright (C) 1997 Free Software Foundation, Inc.

   Written by: ANOQ of the Sun <anoq@vip.cybercity.dk>
   Date: November 1997
   
   This file is part of NSXKit

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "NSStringDrawing.h"
#include "PXKDrawing.h"
#include "NSView.h"
#include "NSAttributedString.h"
#include <Foundation/NSException.h>

@implementation NSString (NSStringAdditions)

- (void)drawAtPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
  NSColor *color;
  NSFont *font;
  NSView *fView;
  NSPoint truePos = aPoint;
  BOOL flipped;
  int fontHeight,tmpBufferLoc,strBegin,strEnd;
  char *strPtr;
  NSString *tmpStr;
  int baseLineToBottom = 3;//This is the distance from the font-baseline to
                           //where the font should be drawn when isFlipped == NO

  fView = [NSView focusView];
  if(fView == nil)
  {
    [NSException raise:NSGenericException
      format:@"No view in focus in class NSString (NSStringAdditions) in method - drawAtPoint:withAttributes:"];
  }

  color = (NSColor *)[attributes objectForKey:
    NSForegroundColorAttributeName];
  if(color==nil)
    color = [NSColor blackColor];
  [color set];

  font = (NSFont *)[attributes objectForKey:
    NSFontAttributeName];
  if(font==nil)
    font = (NSFont *)[NSFont fontWithName:@"Helvetica" size:12];
  [font set];

  flipped = [fView isFlipped];
  fontHeight = [font pointSize];
  if(!flipped)
  {
    truePos.y = [fView frame].size.height - aPoint.y - baseLineToBottom;
  }

  strPtr = (char *)[self cString];
  strBegin = 0;
  strEnd = [self length];
  tmpBufferLoc = 0;
  while(tmpBufferLoc<strEnd)
  {
    while(*strPtr!='\n' && tmpBufferLoc<strEnd)
    {
      tmpBufferLoc++;
      strPtr++;
    }
    tmpStr = [self substringFromRange:NSMakeRange(strBegin,tmpBufferLoc-strBegin)];
    pxkDrawString([tmpStr cString],truePos);
    if(flipped)
      truePos.y += fontHeight;
    else
      truePos.y -= fontHeight;
    tmpBufferLoc++;//Skip '/n'
    strPtr++;//Skip '/n'
    strBegin = tmpBufferLoc;
  }
}

- (void)drawInRect:(NSRect)aRect withAttributes:(NSDictionary *)attributes
{
  [self drawAtPoint:aRect.origin withAttributes:attributes];
  //FIXME: This method must make linefeeds (upwards if not
  //       focusedView isFlipped) using the width of aRect
  //       to determine the length of each line. The height
  //       of aRect is not used as far as I know.
}

- (NSSize)sizeWithAttributes:(NSDictionary *)attributes
{
  NSString *tmpStr;
  NSSize size;
  int fontHeight,strBegin,strEnd,tmpBufferLoc,lineWidth;
  char *strPtr;
  NSFont *font;
 
  strEnd = [self length];
  if(strEnd<=0)
    return NSZeroSize;
  
  font = [attributes objectForKey:NSFontAttributeName];
  if(font==nil)
    font = (NSFont *)[NSFont fontWithName:@"Helvetica" size:12];
  
  strPtr = (char *)[self cString];
  strBegin = 0;
  tmpBufferLoc = 0;
  fontHeight = [font pointSize];
  size = NSMakeSize(0,fontHeight);
  while(tmpBufferLoc<strEnd)
  {
    while(*strPtr!='\n' && tmpBufferLoc<strEnd)
    {
      tmpBufferLoc++;
      strPtr++;
    }
    tmpStr = [self substringFromRange:NSMakeRange(strBegin,tmpBufferLoc-strBegin)];
    lineWidth = [font widthOfString:self];
    if(lineWidth>size.width)
      size.width = lineWidth;
    size.height += fontHeight;
    tmpBufferLoc++;//Skip '/n'
    strPtr++;//Skip '/n'
    strBegin = tmpBufferLoc;
  }
  return size;
}

@end//implementation NSString (NSStringAdditions)