/*
 * Filename: NSTextField.m
 * Project:  Pascal's X kit
 * Module:   Basic application components
 *
 * Function: This is the TextField class
 *
 * Author:   Pascal Forget
 * Date:     March 1996
 *
 * Copyright (C) 1996 Free Software Foundation, Inc.
 */

#include "NSTextField.h"

@implementation NSTextField

- (BOOL)acceptsFirstResponder;
{
    if ((isEditable == YES) || (isSelectable == YES))
    {
	return YES;
    }
    return NO;
}

- (SEL)errorAction;
{
    return errorAction;
}

- init;
{
    [super init];

    isBeingEdited = NO;
    disabled = NO;
    editable = YES;
    stringValue = (char *)malloc(7);
    strcpy(stringValue, "Text");
    
    return self;
}

- display;
{
    [super display];

    XClearWindow(pxkDisplay, xWindow);
    
    if (disabled == NO) {
	XDefineCursor(pxkDisplay,xWindow,
		      XCreateFontCursor(pxkDisplay, XC_xterm));
    }

    [self drawSelf];
    [self drawBorder];
    return self;
}
    
- drawSelf;
{
    int len1;
    int width;
    int font_height;

    /* need length for both XTextWidth and XDrawString */
    len1 = strlen(stringValue);

    if (!win_font) {
	fprintf(stderr, "Error loading font. Exiting.\n");
	exit(-1);
    }
    
    /* get string widths for centering */
    width = XTextWidth(win_font, stringValue, len1);

    font_height = win_font->ascent + win_font->descent;

    /*
     * Wipe out the contents of the window by filling its
     * area with the backGroundColor (defined in PXKView.h)
     */
    XFillRectangle(pxkDisplay, xWindow, pxkWhiteGC,
		   frame.origin.x+2, frame.origin.y+2,
		   frame.size.width-4, frame.size.height-4);

    [self drawBorder];

    if((alignment == PXKALIGN_LEFT) && (width < frame.size.width)) {
	XDrawString(pxkDisplay, xWindow, pxkBlackGC, 5, font_height,
		    stringValue, len1);
    } else if((alignment == PXKALIGN_RIGHT) && ((width+5)<frame.size.width)) {
	XDrawString(pxkDisplay, xWindow, pxkBlackGC,
		    frame.size.width - 5 - width, font_height,
		    stringValue, len1);
    } else {

	if (width > (frame.size.width-4)) {
	    XDrawString(pxkDisplay, xWindow, pxkBlackGC,
			(frame.size.width - width-5), 
			font_height, stringValue, len1);	    
	} else {
	    /* output text, centered on each line */
	    XDrawString(pxkDisplay, xWindow, pxkBlackGC,
			(frame.size.width - width-6)/2, 
			font_height, stringValue, len1);
	}
    }
    
    return self;
}

- drawBorder;
{
    if (!win_font) {
	load_font(&win_font);
    }
    
    XDrawLine(pxkDisplay, xWindow, pxkBlackGC, 0, 0, frame.size.width,0);
    XDrawLine(pxkDisplay, xWindow, pxkBlackGC, 0, 0, 0, frame.size.height);

    XFlush(pxkDisplay);

    /* Draw the bottom and right borders */
    XDrawLine(pxkDisplay, xWindow, pxkLightgrayGC, 2, frame.size.height-2,
	      frame.size.width,frame.size.height-2);
    
    XDrawLine(pxkDisplay, xWindow, pxkLightgrayGC, frame.size.width-2, 2,
	      frame.size.width-2, frame.size.height-2);

    return self;
}

- (BOOL)isBezeled;
{
    delegate = nil;
    return [cell isEditable];
}

- (BOOL)isBordered;
{
    return [cell isBordered];
}

- (BOOL)isEditable;
{
    return [cell isEditable];
}

- (BOOL)isSelectable;
{
    return [cell isSelectable];
}

- (int)intValue;
{
    if ((stringValue != NULL) && (strlen(stringValue) > 0)) {
	return atoi(stringValue);
    } else {
	return 0;
    }
}
    
- (double)floatValue;
{
    return [self doubleValue];
}

- (double)doubleValue;
{
    if ((stringValue != NULL) && (strlen(stringValue) > 0)) {
	return atof(stringValue);
    } else {
	return 0.0;
    }
}

/*
 * Get characters until you encounter
 * a carriage return, return number of characters.
 * Deal with backspaces, etc.  Deal with Expose events
 * on all windows associated with this application.
 * Deal with keyboard remapping.
 */

- keyDown:(XEvent *)e;
{
    XEvent report;
    KeySym keysym;
    XComposeStatus compose;    
    int count, length;
    char buffer[MAX_MAPPED_STRING_LENGTH];
    int bufsize=MAX_MAPPED_STRING_LENGTH;

    report = *e;    
    fprintf(stdout, "PXKTextField keyDown:\n");
    
    count = XLookupString(&report, buffer, bufsize, &keysym, &compose);

    /*
     * now need to do the right thing with
     * every keysym possibility, as minimum:
     */
    if ((keysym == XK_Return) ||
	(keysym == XK_KP_Enter) ||
	(keysym == XK_Linefeed))
    {
	isBeingEdited = NO;
    } else if (((keysym >= XK_KP_Space) && (keysym <= XK_KP_9))
	       ||
	       ((keysym >= XK_space) && (keysym <= XK_asciitilde)))
    {
	if ((strlen(stringValue) + count) >= 2048)
	{
	    XBell(pxkDisplay, 0);
	} else {
	    if (stringValue!= NULL) {
		stringValue = realloc(stringValue,
				      strlen(stringValue) + count + 1);
	    } else {
		stringValue = malloc(count + 1);
	    }
	    (void)strncat(stringValue, buffer, count);
	}
    }
    else if ((keysym >= XK_Shift_L) && (keysym <= XK_Hyper_R))
	;/* do nothing because its a modifier key */
    else if ((keysym >= XK_F1) && (keysym <= XK_F35))
	if (buffer == NULL)
	    printf("Unmapped function key\n");
	else if ((strlen(stringValue) + count)>= MAX_POPUP_STRING_LENGTH)
	{
	    XBell(pxkDisplay, 0);
	} else {
	    if (stringValue!= NULL) {
		stringValue = realloc(stringValue,
				      strlen(stringValue) + count + 1);
	    } else {
		stringValue = malloc(count + 1);
	    }	    
	    (void)strncat(stringValue, buffer, count);
	}
    
    else if ((keysym == XK_BackSpace) || (keysym == XK_Delete)) {
	if ((length = strlen(stringValue)) > 0) {
	    stringValue[length - 1] = NULL;
	    XClearWindow(pxkDisplay, xWindow);
	} else {
	    XBell(pxkDisplay, 0);
	}
    }
    else {
	printf("keysym %s is not handled\n",
	       XKeysymToString(keysym));
	XBell(pxkDisplay, 0);
    }
    [self display];
    /* clear buffer so multi-character strings
     * are properly supported, even though
     * they can only be deleted one char at a
     * time */
    buffer[1] = NULL;

    return self;
}

- mouseDown:(XEvent *)e;
{   
    fprintf(stdout, "PXKTextField mouseDown:\n");
    
    if (!isBeingEdited) {
	XSelectInput(pxkDisplay, xWindow, (ExposureMask | KeyPressMask));
	[self display];
	isBeingEdited = YES;
    } else {
	/* Nothing to do (until selection is implemented) */
    }
    return self;
}

- (void)setBezeled:(BOOL)flag;
{
    [cell setBezeled:flag];
}

- (void)setBordered:(BOOL)flag;
{
    [cell setBordered:flag];
}

- (void)setEditable:(BOOL)flag;
{
    [cell setEditable:flag];
}

- (void)setErrorAction:(SEL)aSelector;
{
    errorAction = aSelector;
}

- (void)setSelectable:(BOOL)flag;
{
    [cell setSelectable:flag];
}

@end
