/* 
   NSTextPrivate.m

   Private part of the NSText class

   Copyright (C) 1997 Free Software Foundation, Inc.

   Author: ANOQ of the Sun <anoq@vip.cybercity.dk>
   Date: November 1997
   
   This file is part of the NSXKit.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include "NSTextPrivate.h"
#include "PXKDrawing.h"
#include <Foundation/NSException.h>

  //Private function for freeing an NSTextChunk - why isn't this a part of
  //the OpenStep specification?
void NSChunkZoneFree(NSTextChunk *pc,NSZone *zone)
{
  NSZoneFree(zone,pc);
}

  //Private function for freeing an NSTextChunk - why isn't this a part of
  //the OpenStep specification?
void NSChunkFree(NSTextChunk *pc)
{
  NSChunkZoneFree(pc,NSDefaultMallocZone());
}

int memLeakTester = 0;

//Private functions for dealing with NSTextBlocks
NSTextBlock *allocNSTextBlock(void)
{
  NSTextBlock *newBlock;
  
  newBlock = objc_malloc(sizeof(NSTextBlock)+NSTextBlockSize);//FIXME:does this raise an exception automatically?
  newBlock->tbFlags.malloced = YES;
  newBlock->tbFlags.PAD = 0;
  newBlock->text = (char *)newBlock+sizeof(NSTextBlock);

  memLeakTester++;
  return newBlock;
}

void freeNSTextBlock(NSTextBlock *block)
{
  OBJC_FREE(block);
  memLeakTester--;
  if(!memLeakTester)
    puts("all NSTextBlocks has now been freed!");
  if(memLeakTester<0)
    puts("too many NSTextBlocks has been freed!");
}

void setStringInNSTextBlock(NSTextBlock *block,char *str,short size)
{
  NSCParameterAssert(block!=NULL);
  NSCParameterAssert(str!=NULL);
  NSCParameterAssert(size<NSTextBlockSize);
  memmove(block->text,str,size);
  block->text[size] = '\0';
  block->chars = size;
}

void insertNSTextBlockBetween(NSTextBlock *newBlock,NSTextBlock *beforeBlock,NSTextBlock *afterBlock)
{
  NSCParameterAssert(newBlock!=NULL);
  
  if(beforeBlock!=NULL)
  {
    beforeBlock->next = newBlock;
    newBlock->prior = beforeBlock;
  }
  else
    newBlock->prior = NULL;
  if(afterBlock!=NULL)
  {
    afterBlock->prior = newBlock;
    newBlock->next = afterBlock;
  }
  else
    newBlock->next = NULL;
}

void removeNSTextBlock(NSTextBlock *block)
{
  NSCParameterAssert(block!=NULL);
  
  if(block->next!=NULL)
    block->next->prior = block->prior;
  if(block->prior!=NULL)
    block->prior->next = block->next;
}

void freeListFromNSTextBlock(NSTextBlock *block)
{
  NSTextBlock *nextBlock = block;
  NSTextBlock *tmpBlock;

  if(block)
  {
    if(block->prior)
      block->prior->next = NULL;
  }
  while(nextBlock)
  {
    tmpBlock = nextBlock;
    nextBlock = tmpBlock->next;
    freeNSTextBlock(tmpBlock);
  }
}

  //Private functions for NSLayInfo...
NSLayInfo *allocNSLayInfo(void)
{
  NSLayInfo *newLayInfo=NULL;
  
  newLayInfo = NSZoneMalloc(NSDefaultMallocZone(),sizeof(NSLayInfo));
  NS_DURING
    memset(newLayInfo,sizeof(newLayInfo),0);
    newLayInfo->lays = (NSLayArray *)NSChunkMalloc(sizeof(NSLay)*16,0);
    NS_DURING
      newLayInfo->widths = (NSWidthArray *)NSChunkMalloc(sizeof(float)*16,0);
      NS_DURING
        newLayInfo->chars = (NSCharArray *)NSChunkMalloc(sizeof(unsigned char)*16,0);
      NS_HANDLER
        NSChunkFree((NSTextChunk *)newLayInfo->widths);
        [localException raise];
      NS_ENDHANDLER
    NS_HANDLER
      NSChunkFree((NSTextChunk *)newLayInfo->lays);
      [localException raise];
    NS_ENDHANDLER
  NS_HANDLER
    NSZoneFree(NSDefaultMallocZone(),newLayInfo);
    newLayInfo = NULL;//This should not be nessecary - the function will not return...
    [localException raise];
  NS_ENDHANDLER
  return newLayInfo;
}

void freeNSLayInfo(NSLayInfo *layInfo)
{
  if(!layInfo)
    return;
  
  if(layInfo->chars)
    NSChunkFree((NSTextChunk *)layInfo->chars);
  if(layInfo->widths)
    NSChunkFree((NSTextChunk *)layInfo->widths);
  if(layInfo->lays)
    NSChunkFree((NSTextChunk *)layInfo->lays);
  NSZoneFree(NSDefaultMallocZone(),layInfo);
}

  //Adds an NSLineDesc to breakArray and does a realloc if nessecary
void pxkAddBreak(NSBreakArray *breakArray,NSLineDesc lineDesc)
{
  if(breakArray->chunk.used+sizeof(NSLineDesc)>breakArray->chunk.allocated)
    NSChunkRealloc((NSTextChunk *)breakArray);
  breakArray->breaks[ breakArray->chunk.used/sizeof(NSLineDesc) ] = lineDesc;
  breakArray->chunk.used += sizeof(NSLineDesc);
}

  //Calculates linebreaks for an NSTextBlock-list
int pxkCalcLineBreaks(NSLayInfo *layInfo,NSRun *run,NSTextBlock *firstTextBlock,NSBreakArray *breakArray)
{
  NSTextBlock *block = firstTextBlock;
  int blockPos = 0;

  NSLineDesc lineDesc;
  NSLineDesc widthDesc;
  int lineWidth,tmpWidth,maxLineWidth;
  char *widthPtr;
  NSLineDesc possibleBreak;
  int possibleWidth;
  NSString *tmpStr;
  char curChar;
  BOOL result = 0;

  breakArray->chunk.used = 0;
  maxLineWidth = layInfo->rect.size.width;
  lineDesc = 0;
  widthDesc = 0;
  lineWidth = 0;
  possibleBreak = 0;
  possibleWidth = 0;
  while(block!=NULL)
  {
    widthPtr = block->text;
    while(blockPos < block->chars)
    {
      curChar = block->text[blockPos];
      lineDesc++;
      blockPos++;
      if(curChar==' ' || curChar=='\t' || curChar=='\n')
      {
        tmpStr = [NSString stringWithCString:widthPtr
                           length:(lineDesc-widthDesc)];
        [tmpStr autorelease];
        tmpWidth = pxkWidthOfString(tmpStr,run->font);
        widthPtr += (lineDesc-widthDesc);
        if(tmpWidth+lineWidth > maxLineWidth)
        {
          if(possibleBreak>0)
          {
            pxkAddBreak(breakArray,possibleBreak);
            //widthPtr += possibleBreak;
            lineDesc -= possibleBreak;
            lineWidth -= possibleWidth;
            lineWidth += tmpWidth;
          }
          if(possibleBreak==0 || lineWidth > maxLineWidth)
          {
            result = 1;//Truncate a word... FIXME: check charwrap...
            pxkAddBreak(breakArray,lineDesc);
            lineDesc = 0;
            lineWidth = 0;
          }
          else if(curChar=='\n')
          {
            //Force line feed
            pxkAddBreak(breakArray,lineDesc);
            lineDesc = 0;
            lineWidth = 0;
          }
          possibleBreak = 0;//possibleWidth does not matter when possibleBreak == 0
        }
        else
        {
          if(curChar=='\n')
          {
            //Force linefeed
            pxkAddBreak(breakArray,lineDesc);
            lineDesc = 0;
            lineWidth = 0;
          }
          else
          {
            //The is a possible break
            //widthPtr += (lineDesc-widthDesc);
            lineWidth += tmpWidth;
            possibleBreak = lineDesc;
            possibleWidth = lineWidth;
          }
        }
        widthDesc = lineDesc;
      }
    }
    block = block->next;
    blockPos = 0;
  }
  if(lineDesc>0)
    pxkAddBreak(breakArray,lineDesc);
  
  return result;
}

  //Calculates linebreaks for an NSTextStorage
int pxkCalcTextStorageLineBreaks(NSTextStorage *textStorage,NSBreakArray *breakArray,NSSize size)
{
  int textPos,textLength;
  NSLineDesc lineDesc;
  NSLineDesc widthDesc;
  int lineWidth,tmpWidth,tmpTmpWidth,maxLineWidth,textPieceBegin,widthPos;
  char *textPtr;
  NSLineDesc possibleBreak;
  int possibleWidth;
  NSString *tmpStr;
  char curChar;
  BOOL result = 0;
  NSRange effectiveRange = NSMakeRange(0,0);
  NSDictionary *attrs;
  NSFont *font;

  breakArray->chunk.used = 0;
  maxLineWidth = size.width;
  lineDesc = 0;
  widthDesc = 0;
  lineWidth = 0;
  possibleBreak = 0;
  possibleWidth = 0;
  textLength = [textStorage length];
  textPtr = (char *)[[textStorage string] cString];
  textPos = 0;
  widthPos = 0;
  tmpTmpWidth = 0;
  while(NSMaxRange(effectiveRange) < textLength)
  {
    attrs = [textStorage attributesAtIndex:NSMaxRange(effectiveRange)
      effectiveRange:&effectiveRange];
    font = (NSFont *)[attrs objectForKey:NSFontAttributeName];
    if(font==nil)
      font = (NSFont *)[NSFont fontWithName:@"Helvetica" size:12];

    textPieceBegin = effectiveRange.location;
    for(textPos=effectiveRange.location;
          textPos<NSMaxRange(effectiveRange);
          textPos++)
    {
      curChar = textPtr[textPos];
      lineDesc++;
      if(curChar==' ' || curChar=='\t' || curChar=='\n')
      {
        tmpStr = [[textStorage string] substringFromRange:
                  NSMakeRange(textPieceBegin,(textPos-textPieceBegin))];
        tmpWidth = tmpTmpWidth + [font widthOfString:tmpStr];
        tmpTmpWidth = 0;
        textPieceBegin = textPos;
        if(tmpWidth+lineWidth > maxLineWidth)
        {
          if(possibleBreak>0)
          {
            pxkAddBreak(breakArray,possibleBreak);
            //widthPtr += possibleBreak;
            lineDesc -= possibleBreak;
            lineWidth -= possibleWidth;
            lineWidth += tmpWidth;
          }
          if(possibleBreak==0 || lineWidth > maxLineWidth)
          {
            result = 1;//Truncate a word... FIXME: check charwrap...
            pxkAddBreak(breakArray,lineDesc);
            lineDesc = 0;
            lineWidth = 0;
          }
          else if(curChar=='\n')
          {
            //Force line feed
            pxkAddBreak(breakArray,lineDesc);
            lineDesc = 0;
            lineWidth = 0;
          }
          possibleBreak = 0;//possibleWidth does not matter when possibleBreak == 0
        }
        else
        {
          if(curChar=='\n')
          {
            //Force linefeed
            pxkAddBreak(breakArray,lineDesc);
            lineDesc = 0;
            lineWidth = 0;
          }
          else
          {
            //The is a possible break
            //widthPtr += (lineDesc-widthDesc);
            lineWidth += tmpWidth;
            possibleBreak = lineDesc;
            possibleWidth = lineWidth;
          }
        }
        widthDesc = lineDesc;
      }
    }
    if(textPieceBegin != textPos)
    {
      tmpStr = [[textStorage string] substringFromRange:
                NSMakeRange(textPieceBegin,(textPos-textPieceBegin))];
      tmpTmpWidth += [font widthOfString:tmpStr];
      textPieceBegin = textPos;
    }
  }
  if(lineDesc>0)
    pxkAddBreak(breakArray,lineDesc);
  
  return result;
}