/* 
   NSTextStorage.m

   Implementation of textstorage-class

   Copyright (C) 1997 Free Software Foundation, Inc.

   Written by: ANOQ of the Sun <anoq@vip.cybercity.dk>
   Date: November 1997
   
   This file is part of NSXKit

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "NSTextStorage.h"
#include "NSGTextStorage.h"

//FIXME: This implementation is NOT complete

static Class NSTextStorage_concrete_class;

@implementation NSTextStorage

+ (void) _setTextStorageConcreteClass: (Class)c
{
  NSTextStorage_concrete_class = c;
}

+ (Class) _concreteTextStorageClass
{
  return NSTextStorage_concrete_class;
}

+ allocWithZone: (NSZone*)z
{
  return NSAllocateObject([self _concreteTextStorageClass], 0, z);
}

//NSMutableCopying protocol
- mutableCopyWithZone: (NSZone*)zone
{
  return [[[[self class] _concreteTextStorageClass] allocWithZone:zone]
	  initWithAttributedString:self];
}

+ (void) initialize
{
  if (self == [NSTextStorage class])
  {
    NSTextStorage_concrete_class = [NSGTextStorage class];
  }
}

- (void)dealloc
{
  if(_layoutManagers)
    [_layoutManagers release];
  [super dealloc];
}

-(void)addLayoutManager:(NSLayoutManager *)aLayoutManager
{
  if(!_layoutManagers)
    _layoutManagers = [[NSMutableArray alloc] init];
  [_layoutManagers addObject:aLayoutManager];
}

-(void)removeLayoutManager:(NSLayoutManager *)aLayoutManager
{
  if(_layoutManagers)
    [_layoutManagers removeObjectIdenticalTo:aLayoutManager];
}

- (NSArray *)layoutManagers
{
  if(_layoutManagers)
    return [[_layoutManagers copy] autorelease];
  else
    return [NSArray array];
}

- (int)changeInLength
{
  return _changeInLength;
}
  
-(void)edited:(unsigned)mask range:(NSRange)oldRange changeInLength:
  (int)lengthChange
{
  int difference;

  if(_editedRange.length == 0)
  {
    _editedRange =
      NSMakeRange(
        oldRange.location,
        oldRange.length + lengthChange);
  }
  else
  {
    difference = _editedRange.location - oldRange.location;
    if(difference > 0)
    {
      _editedRange.length += difference;
      _editedRange.location -= difference;
    }

    difference = NSMaxRange(oldRange) + lengthChange - NSMaxRange(_editedRange);
    if(difference > 0)
      _editedRange.length += difference;
  }

  _changeInLength += lengthChange;
  _editedMask |= mask;

  if(!_editNest)
    [self processEditing];
}

- (unsigned int)editedMask
{
  return _editedMask;
}

- (NSRange)editedRange
{
  return _editedRange;
}

- (void)beginEditing
{
  _editNest ++;
}

-(void)endEditing
{
  if(_editNest <= 0)
    return;
   
  _editNest--;
  if(!_editNest)
    [self processEditing];
}

-(void)processEditing
{
  /*Cleans up changes made to the receiver and notifies its delegate and layout
  managers of changes. This method is automatically invoked in response to an
  endEditing or edited:range:changeInLength: message. You should never need to
  invoke it directly.

  This method begins by posting an NSTextStorageWillProcessEditingNotification
  to the default notification center (which results in the delegate receiving a
  textStorageWillProcessEditing: message). It then invokes the inherited
  fixAttributesAfterEditingRange: method to fix up attributes after a batch of
  editing changes. After this, it posts an NSTextStorageDidProcessEditingNotification
  to the default notification center (which results in the delegate receiving a
  textStorageDidProcessEditing: message). Finally, it sends a
  textStorage:edited:range:changeInLength:invalidatedRange: message to each of the
  receiver's NSLayoutManagers using the argument values provided.*/

  //FIXME: Is this the correct place for this?
  _editedRange = NSMakeRange(0,0);
  _editedMask = 0;
  _changeInLength = 0;
}

-(void)setDelegate:(id)anObject
{
  //FIXME: Should it be retained/released?
  if(_delegate)
    [_delegate release];
  _delegate = anObject;
  if(_delegate)
    [_delegate retain];
}

-(id)delegate
{
  return _delegate;
}

@end //implementation NSTextStorage

/*
Methods Implemented By the Delegate

 textStorageDidProcessEditing:

-(void)textStorageDidProcessEditing:(NSNotification *)aNotification

Informs the delegate that an NSTextStorage object has finished processing edits.
The text storage object is available by sending object to aNotification, which is
always an NSTextStorageDidProcessEditingNotification. The delegate can use this
notification to verify the final state of the text storage object; it can't
change the text storage object's characters without leaving it in an inconsistent
state, but if necessary it can change attributes. Note that even in this case
it's possible to put a text storage object into an inconsistent statefor
example by changing the font of a range to one that doesn't support the
characters in that range (such as using a Latin font for Kanji text).



6 Classes: NSTextStorage

 textStorageWillProcessEditing:

-(void)textStorageWillProcessEditing:(NSNotification *)aNotification

Informs the delegate that an NSTextStorage object is about to process edits.
The text storage object is available by sending object to aNotification, which
is always an NSTextStorageWillProcessEditingNotification. The delegate can use
this notification to verify the changed state of the text storage object, and to
make changes to the text storage object's characters or attributes to enforce
whatever constraints it establishes (which doesn't result in this message being
sent again, however). For example, a code editor application might add a
delegate that checks after edits to make sure that all programming language
keywords are set in boldface.

Notifications

NSTextStorageDidProcessEditingNotification

Posted after the NSTextStorage finishes processing edits in processEditing.
Observers other than the delegate shouldn't make further changes to the
NSTextStorage. The notification contains:

Notification Object The NSTextStorage that processed edits.

Userinfo None

NSTextStorageWillProcessEditingNotification

Posted before the NSTextStorage finishes processing edits in processEditing.
Observers other than the delegate shouldn't make further changes to the
NSTextStorage. The notification contains:

Notification Object The NSTextStorage about to process edits.

Userinfo None
*/