/*
 * Filename: PXKCell.m
 * Project:  Pascal's X Kit
 * Module:   Basic components
 *
 * Function: See header file
 *
 * Author:   Pascal Forget
 * Date:     October 1995
 *
 * Copyright (C) 1995 Pascal Forget. All Rights Reserved.
 */

#include "PXKCell.h"
#include "PXKTextField.h"

@implementation PXKCell

+ initialize;
{
    return self;
}

- init;
{
    [super init];
    stringValue = nil;
    _controlView = nil;
    isOpaque = YES;
    state = 0;
    return self;
}

- (SEL)action;
{
    return action;
}

- (PXKView *)controlView;
{
    return _controlView;
}

- (void)drawBorder;
{
    /* Not implemented yet */
}

- (void)drawImage;
{
    /* Not implemented yet */
}

- (void)drawTextInRect:(PXKRect)frame;
{
    /* Goofy implementation for testing only */
    
    int len1;
    int width;
    int font_height;
    int alignment = 0;
    XFontStruct *win_font;
#if 0
    fprintf(stdout, "Cell text frame: %d %d %d %d\n",
	    frame.origin.x, frame.origin.y,
	    frame.size.width, frame.size.height);
#endif    
    (void)load_font(&win_font);
    
    /* need length for both XTextWidth and XDrawString */
    len1 = [stringValue length];

    /* get string widths for centering */
    width = XTextWidth(win_font, [stringValue cString], len1);

    font_height = win_font->ascent + win_font->descent;

    /*
     * Wipe out the contents of the window by filling its
     * area with the backGroundColor (defined in PXKView.h)
     */

    if (isOpaque) {
	XFillRectangle(pxkDisplay, [_controlView xWindow], pxkLightgrayGC,
		       frame.origin.x+2, frame.origin.y+2,
		       frame.size.width-4, frame.size.height-4);
    }
    
    if((alignment == PXKALIGN_LEFT) && (width < frame.size.width)) {
	XDrawString(pxkDisplay, [_controlView xWindow],
		    pxkBlackGC,
		    frame.origin.x + 5,
		    frame.origin.y + font_height,
		    [stringValue cString], len1);
    } else if((alignment == PXKALIGN_RIGHT) && ((width+5)<frame.size.width)) {
	XDrawString(pxkDisplay, [_controlView xWindow],
		    pxkBlackGC,
		    frame.origin.x + frame.size.width - 5 - width,
		    frame.origin.y + font_height,
		    [stringValue cString], len1);
    } else {

	if (width > (frame.size.width-4)) {
	    XDrawString(pxkDisplay, [_controlView xWindow],
			pxkBlackGC,
			frame.origin.x + frame.size.width - width-5, 
			frame.origin.y + font_height,
			[stringValue cString], len1);	    
	} else {
	    /* output text, centered on each line */
	    XDrawString(pxkDisplay, [_controlView xWindow],
			pxkBlackGC,
			frame.origin.x + (frame.size.width - width-6)/2, 
			frame.origin.y + font_height,
			[stringValue cString], len1);
	}
    }
}

- (void)drawInteriorWithFrame:(PXKRect)cellFrame inView:(PXKView *)controlView;
{
    _controlView = controlView;

    if (image) {
	[self drawImage];
    }

    if (stringValue) {
	[self drawTextInRect:[self titleRectForBounds:cellFrame]];
    }
}

- (void)drawWithFrame:(PXKRect)cellFrame inView:(PXKView *)controlView;
{
    _controlView = controlView;

    [self drawBorder];
    [self drawInteriorWithFrame:cellFrame inView:controlView];
}

- (BOOL)isBezeled;
{
    return isBezeled;
}

- (BOOL)isBordered;
{
    return isBordered;
}

- (BOOL)isContinuous;
{
    return isContinuous;
}

- (BOOL)isEditable;
{
    return isEditable;
}

- (BOOL)isEnabled;
{
    return isEnabled;
}

- (BOOL)isOpaque;
{
    return isOpaque;
}

- (BOOL)isScrollable;
{
    return isScrollable;
}

- (BOOL)isSelectable;
{
    return isSelectable;
}

- (void)setAction:(SEL)aSelector;
{
    action = aSelector;
}

- (void)setBezeled:(BOOL)flag;
{
    isBezeled = flag;
}

- (void)setBordered:(BOOL)flag;
{
    isBordered = flag;
}

- (void)setContinuous:(BOOL)flag;
{
    isContinuous = flag;
}

- (void)setEditable:(BOOL)flag;
{
    isEditable = flag;
}

- (void)setEnabled:(BOOL)flag;
{
    isEnabled = flag;
}

- (void)setOpaque:(BOOL)flag;
{
    isOpaque = flag;
}

- (void)setScrollable:(BOOL)flag;
{
    isScrollable = flag;
}

- (void)setSelectable:(BOOL)flag;
{
    isSelectable = flag;
}

- (void)setState:(int)value;
{
    if ((value == 0) || (value == 1)) {
	state = value;
    }
}

- (void)setStringValue:(PXKString *)aString;
{
    if (stringValue != nil) {
	[stringValue release];
    }

    stringValue = [aString retain];
}

- (void)setTag:(int)anInt;
{
    tag = anInt;
}

- (void)setTarget:(id)anObject;
{
    target = anObject;
}

- (void)setWraps:(BOOL)flag;
{
    wraps = flag;
}

- (int)state;
{
    return state;
}

- (PXKString *)stringValue;
{
    return stringValue;
}

- (int)tag;
{
    return tag;
}

- (id)target;
{
    return target;
}

- (PXKRect)titleRectForBounds:(PXKRect)theRect;
{
#if 0
    fprintf(stdout, "Cell titleRectForBounds: %d %d %d %d\n",
	    theRect.origin.x, theRect.origin.y,
	    theRect.size.width, theRect.size.height);
#endif        
    /* Goofy implementation for testing only */
    return theRect;
}

- (BOOL)wraps;
{
    return wraps;
}

@end
