/* 
   testNSImage.m

   Small example to test the NSImage and related classes

   Copyright (C) 1997 Free Software Foundation, Inc.

   Written by: ANOQ of the sun <anoq@vip.cybercity.dk>
   Date: November 1997
   
   This file is part of NSXKit

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "NSWindow.h"
#include "NSApplication.h"
#include "NSBitmapImageRep.h"
#include "NSImage.h"

void testNSImage(NSWindow *win)
{
    NSImage *anImage;
    NSRect tmpRect;

    anImage = [[NSImage alloc] initByReferencingFile:@"TiffTest.tiff"];
    [anImage autorelease];
    [[win contentView] lockFocus];
    tmpRect.origin.x = 0;
    tmpRect.origin.y = 0;
    [anImage drawRepresentation:[[anImage representations] objectAtIndex:0] inRect:tmpRect];
    [[win contentView] unlockFocus];
}

void testNSBitmapRep(NSWindow *win)
{
    NSBitmapImageRep *aBitmapRep;
    NSRect tmpRect;

    //Load image into bitmap-rep
    aBitmapRep = [NSBitmapImageRep imageRepWithContentsOfFile:@"TiffTest.tiff"];
    [aBitmapRep autorelease];

    //display image
    tmpRect.origin.x = 0;
    tmpRect.origin.y = 0;
    [[win contentView] lockFocus];
    [aBitmapRep drawAtPoint:tmpRect.origin];

    //grab some of the image
    tmpRect.origin.x = 20;
    tmpRect.origin.y = 20;
    tmpRect.size.width = 40;
    tmpRect.size.height = 40;
    aBitmapRep = [[NSBitmapImageRep alloc] initWithFocusedViewRect:tmpRect];
    [aBitmapRep autorelease];

    //display grabbed image in 3 locations
    tmpRect.origin.x = 0;
    tmpRect.origin.y = 100;
    [aBitmapRep drawAtPoint:tmpRect.origin];
    tmpRect.origin.x = 80;
    tmpRect.origin.y = 100;
    [aBitmapRep drawAtPoint:tmpRect.origin];
    tmpRect.origin.x = 160;
    tmpRect.origin.y = 100;
    [aBitmapRep drawAtPoint:tmpRect.origin];
    [[win contentView] unlockFocus];
}

int
main(int argc, char **argv)
{
    NSRect rect;
    NSWindow *win;
    NSApplication *app = [NSApplication sharedApplication];

    initializeDrawingEngine();

    /* Create a new window */
    
    rect.origin.x = 0;
    rect.origin.y = 0;
    rect.size.width = 400;
    rect.size.height = 300;

    win = [[NSWindow alloc] initWithContentRect:rect
			    styleMask:0
			    backing:NSBackingStoreBuffered
			    defer:NO];
    [win display];
    
    testNSBitmapRep(win);
    testNSImage(win); // test NSImage

    [app setMainWindow: win];
    [app run];

    return 0;
}