/* 
   testStringDrawing.m

   Small example to test the drawing of strings in the AppKit additions to NSString and NSAttributedString

   Copyright (C) 1997 Free Software Foundation, Inc.

   Written by: ANOQ of the sun <anoq@vip.cybercity.dk>
   Date: November 1997
   
   This file is part of NSXKit

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "NSWindow.h"
#include "NSApplication.h"
#include <Foundation/NSDictionary.h>
#include "NSAttributedString.h"
#include "NSStringDrawing.h"

void testDrawAttributedString(NSView *view)
{
  NSMutableAttributedString *muAttrString;
  NSMutableDictionary *attributes,*colorAttributes;
  
  attributes = [[[NSMutableDictionary alloc] init] autorelease];
  [attributes setObject:[NSFont fontWithName:@"Helvetica" size:20]
    forKey:NSFontAttributeName];
  colorAttributes = [[[NSMutableDictionary alloc] init] autorelease];
  [colorAttributes setObject:[NSColor redColor]
    forKey:NSForegroundColorAttributeName];

  muAttrString = [[NSMutableAttributedString alloc]
    initWithString:@"Testing the Mutable version\nNew line\n- and again\nThe text is drawn UPWARDS because the method \"-isFlipped\" returns NO in this view"];
  puts("alloc'ed");
  [muAttrString autorelease];
  [muAttrString setAttributes:attributes range:NSMakeRange(0,7)];
  puts("setattrs1");
  [muAttrString setAttributes:colorAttributes range:NSMakeRange(12,7)];
  puts("setattrs2");
  [muAttrString setAttributes:attributes range:NSMakeRange(32,10)];
  puts("setattrs3");
  
  [view lockFocus];
  [muAttrString drawAtPoint:NSMakePoint(0,100)];
  [view unlockFocus];
  puts("drawn");
}

void testStringDrawing(NSView *view)
{
    NSString *str;
    NSSize size;
    
    str = @"TextTest... This is some text, which should show off some wordwrapping. And here is some more text to test if it works with many lines of text! And now:\nLinefeed!";
    size = [str sizeWithAttributes:[NSDictionary dictionary]];
    printf("%ld,%ld\n",(long)size.width,(long)size.height);
    
    [view lockFocus];
    [str drawAtPoint:NSMakePoint(0,0)
      withAttributes:[NSDictionary dictionary]];
    [view unlockFocus];
}

int
main(int argc, char **argv)
{
    NSRect rect;
    NSWindow *win;
    NSApplication *app = [NSApplication sharedApplication];

    initializeDrawingEngine();

    /* Create a new window */
    
    rect.origin.x = 0;
    rect.origin.y = 0;
    rect.size.width = 400;
    rect.size.height = 300;

    win = [[NSWindow alloc] initWithContentRect:rect
			    styleMask:0
			    backing:NSBackingStoreBuffered
			    defer:NO];
    
    [win display];

    testStringDrawing([win contentView]);
    testDrawAttributedString([win contentView]);
    
    [app setMainWindow: win];
    [app run];

    return 0;
}