/*
 * Filename: ns_main.m
 * Project:  Pascal's NSX Kit
 *
 * Author:   Pascal Forget
 * Date:     January 1996
 *
 * Copyright (C) 1996 Free Software Foundation
 */
#include "ns_main.h"
#include "PXKList.h"
#include "PXKDrawing.h"
#include "PXKGeometry.h"
#include "NSWindow.h"
#include "NSButtonCell.h"
#include "NSBox.h"
#include "NSButton.h"
#include "NSApplication.h"
#include "NSBitmapImageRep.h"
#include "NSCachedImageRep.h"
#include "NSGlobal.h"
#include "NSText.h"

@interface MyController : NSObject
{
  NSWindow *win;
}

- setWindow:(NSWindow *)newWin;
- (void)printMessage;
@end

@implementation MyController

- setWindow:(NSWindow *)newWin
{
  win = newWin;
}

- (void)printMessage;
{
    puts("MyController says Hello!");
}

@end

void
drawLines(NSWindow *win)
{
    NSPoint a, b;
    NSRect aLine;
    NSRect rect;

    a.x = 10;
    a.y = 5;
    b.x = 100;
    b.y = 50;
    
    [(NSView *)[win contentView] lockFocus];

    aLine.origin.x = a.x;
    aLine.origin.y = a.y;
    aLine.size.width = 90;
    aLine.size.height = 35;
    
    rect.origin.x = 20.0;
    rect.origin.y = 10.0;
    rect.size.width = 300.0;
    rect.size.height = 150.0;    
    
    aLine = clipLineToRect(aLine, rect);
    
    a.x = aLine.origin.x;
    a.y = aLine.origin.y;
    b.x = aLine.origin.x + aLine.size.width;
    b.y = aLine.origin.y + aLine.size.height;
    
    fprintf(stderr, "clipped line: %3.0f %3.0f %3.0f %3.0f\n",
	    a.x, a.y, b.x, b.y);
    
    pxkDrawLine(a,b,[NSColor blackColor]); 

    [(NSView *)[win contentView] unlockFocus];
    [win display];
}

void
testLineIntersection(void)
{
    NSRect rectA, rectB;
    NSPoint intersect;
    
    rectA.origin.x = 100.0;
    rectA.origin.y = 100.0;
    rectA.size.width = 100.0;
    rectA.size.height = 0.0;

    rectB.origin.x = 150.0;
    rectB.origin.y = 100.0;
    rectB.size.width = 0.0;
    rectB.size.height = 100.0;
    
    intersect = intersectionPoint(rectA, rectB);
    
    fprintf(stdout, "intersection: %3.0f %3.0f\n", 
	    intersect.x,
	    intersect.y);
}

void
drawRect(NSWindow *win)
{    
    NSRect rect;
    
    fprintf(stderr, "Drawing rect...\n");    

    rect.origin.x = 20.0;
    rect.origin.y = 10.0;
    rect.size.width = 300.0;
    rect.size.height = 150.0;
    
    [(NSView *)[win contentView] lockFocus];

    NSFrameRect(rect);

    [(NSView *)[win contentView] unlockFocus];
    [win display];
    
    printf("Done.\n");
    sleep(1);
}

void
drawButtons(NSWindow *win)
{
    NSPoint a, b;
    NSRect aLine;
    NSRect rect;

    a.x = 60;
    a.y = 200;
    b.x = 100;
    b.y = 220;
    
    [(NSView *)[win contentView] lockFocus];

    aLine.origin.x = a.x;
    aLine.origin.y = a.y;
    aLine.size.width = 90;
    aLine.size.height = 35;
    
    rect.origin.x = 0.0;
    rect.origin.y = 0.0;
    rect.size.width = 900.0;
    rect.size.height = 950.0;    

    NSDrawButton(aLine, rect); /* here the button fits entirely in clipRect */
    
    aLine.origin.x = 200.0;
    rect.origin.x = 220.0; /* here we'll have to clip */
    NSDrawButton(aLine, rect);
    
    [(NSView *)[win contentView] unlockFocus];
    [win display];
}

void
drawBox(NSWindow *win)
{
    NSRect aLine;
    NSRect rect;

    [(NSView *)[win contentView] lockFocus];

    aLine.origin.x = 60;
    aLine.origin.y = 40;
    aLine.size.width = 100;
    aLine.size.height = 60;
    
    rect.origin.x = 0.0;
    rect.origin.y = 0.0;
    rect.size.width = 900.0;
    rect.size.height = 950.0;    

    NSDrawGroove(aLine, rect);
        
    [(NSView *)[win contentView] unlockFocus];
    [win display];
}


void
drawBezel(NSWindow *win)
{
    NSRect aLine;
    NSRect rect;

    [(NSView *)[win contentView] lockFocus];

    aLine.origin.x = 200;
    aLine.origin.y = 40;
    aLine.size.width = 60;
    aLine.size.height = 60;
    
    rect.origin.x = 100.0;
    rect.origin.y = 0.0;
    rect.size.width = 400.0;
    rect.size.height = 400.0;    

    pxkFillRect(aLine, [NSColor whiteColor]);
    
    NSDrawGrayBezel(aLine, rect);
        
    [(NSView *)[win contentView] unlockFocus];
    [win display];
}


void
addCell(NSWindow *win)
{
    NSRect rect = {{170.0, 5.0}, {84.0, 30.0}};
    NSString *title = [[NSString alloc] initWithCString:"default"];
    NSCell *aCell = [[NSCell alloc] initTextCell: title];

    [aCell setFont: [NSFont fontWithName: @"helvetica" size: 24]];
    
    [(NSView *)[win contentView] lockFocus];

    [aCell drawWithFrame: rect inView: [win contentView]];

    [(NSView *)[win contentView] unlockFocus];
    
    [win display];
}

NSBox *addBox(NSView *view)
{
    NSRect rect = {{150.0, 100.0}, {200.0, 100.0}};
    NSBox *aBox = [[NSBox alloc] initWithFrame: rect];
    [aBox setTitle:@"Box"];
    //[aBox display];
    [view addSubview: aBox];
    [aBox display];
    return aBox;
}

addButton(NSView *view)
{
    MyController *controller = [[MyController alloc] init];
    
    NSRect rect = {{10.0, 10.0}, {100.0, 24.0}};
    NSButton *aButton = [[NSButton alloc] initWithFrame: rect];
    
    [controller setWindow:[view window]];
    [aButton setTarget: controller];
    [aButton setAction: @selector(printMessage)];
    
    [aButton setTitle: @"Hello Button"];
    [view addSubview: aButton];
    [aButton display];
}

NSText *addText(NSView *view)
{
    //The size {150, 150} is meant for the text I have written... And don't
    //forget to call the display method...
    NSRect rect = {{20.0, 20.0}, {150.0, 150.0}};
    NSText *theText = [[NSText alloc] initWithFrame: rect];
    
    [theText setText:@"TextTest... This is some text, which should show off some
 wordwrapping. And here is some more text to test if it works with many lines of
 text! And now:\nLinefeed!"];
    
    [view addSubview: theText];
}


int
main(int argc, char **argv)
{
    NSRect rect;
    int i;
    int windowId;
    NSWindow *win,*tmpWin;
    NSBox *aBox;
    NSApplication *app = [NSApplication sharedApplication];
    NSRect tmpRect;
    NSBitmapImageRep *aBitmapRep;
    //NSCachedImageRep *aCacheRep;
    NSImage *anImage;

    initializeDrawingEngine();

    /* Create a new window */
    
    rect.origin.x = 0;
    rect.origin.y = 0;
    rect.size.width = 640;
    rect.size.height = 480;

    win = [[NSWindow alloc] initWithContentRect:rect
			    styleMask:0
			    backing:NSBackingStoreBuffered
			    defer:NO];
    
    [win setTitle:@"Hi!"];
    
    /* Display the window */

    [win display];

    windowId = [win windowNumber];
    
#if 0
    [win close];

    sleep(2);

    [win display];

    sleep(2);

    /* Resize the window */

    printf("Heightening window\n");

    for (i=0; i<4; i++ ) {
	rect.size.height += 50;
	//setWindowFrameSize(windowId, rect.size);
	//displayWindow(windowId);

	[win setFrame:rect display:YES];

	sleep(1);
    }
    printf("done.\n");    

    printf("Widening window\n");
    
    for (i=0; i<4; i++ ) {
	rect.size.width += 50;
	setWindowFrameSize(windowId, rect.size);
	displayWindow(windowId);
	sleep(1);
    }
    printf("done.\n");
    
#endif

#if 0    
    /* Move and resize at the same time */

    printf("Moving and resizing window\n");
    
    for (i=0; i<200; i+=50) {
	rect.origin.x = 100 + i;
	rect.origin.y = 300 + i;
	rect.size.width = 400 + i;
	rect.size.height = 200 + i;

	[win setFrame:rect display:YES];
	
	//setWindowFrame(windowId, rect);
	//displayWindow(windowId);

    }
    sleep(1);
    printf("done.\n");
    
    /* Draw lines in the content view */

    drawLines(win);


    /* Test the intersection of two 2D lines */

    testLineIntersection();
    
    drawRect(win);

    drawButtons(win);

    drawBox(win);
    
    drawBezel(win);
    
    addCell(win);   // test NSCell

    addBox(win);    // test NSBox
#endif
    aBox = addBox([win contentView]);    // test NSBox
    addButton([win contentView]); // test NSButton
    [win display];
 
    aBitmapRep = [NSBitmapImageRep imageRepWithContentsOfFile:@"TiffTest.tiff"];
    [aBitmapRep autorelease];
    tmpRect.origin.x = 0;
    tmpRect.origin.y = 0;
    //tmpRect.size = [aBitmapRep size];
    //aCacheRep = [[NSCachedImageRep alloc] autorelease];
    //[aCacheRep initWithWindow:nil rect:tmpRect];
    //tmpWin = [aCacheRep window];
    //[[tmpWin contentView] lockFocus];
    //[aBitmapRep drawAtPoint:tmpRect.origin];
    //[[tmpWin contentView] unlockFocus];
    [[win contentView] lockFocus];
    [aBitmapRep drawAtPoint:tmpRect.origin];
    tmpRect.origin.x = 200;
    tmpRect.origin.y = 200;
    tmpRect.size.width = 100;
    tmpRect.size.height = 100;
    aBitmapRep = [[[NSBitmapImageRep alloc] autorelease] initWithFocusedViewRect:tmpRect];
    tmpRect.origin.x = 0;
    tmpRect.origin.y = 0;
    [aBitmapRep drawAtPoint:tmpRect.origin];
    tmpRect.origin.x = 200;
    tmpRect.origin.y = 0;
    [aBitmapRep drawAtPoint:tmpRect.origin];
    tmpRect.origin.x = 400;
    tmpRect.origin.y = 0;
    [aBitmapRep drawAtPoint:tmpRect.origin];
    [[win contentView] unlockFocus];

    
//    anImage = [[[NSImage alloc] autorelease] initByReferencingFile:@"TiffTest.tiff"];
//    [[win contentView] lockFocus];
//    tmpRect.origin.x = 0;
//    tmpRect.origin.y = 0;
//    [anImage drawRepresentation:[[anImage representations] objectAtIndex:0] inRect:tmpRect];
//    [[win contentView] unlockFocus];

    [addText([win contentView]) display];

    [app setMainWindow: win];
    [app run];

    return 0;
}
